<?php

/**
 * Helpers
 *
 * @package     PG_Core
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Validate Email Addresses Via SMTP
 *
 * This queries the SMTP server to see if the email address is accepted.
 *
 * @package         PG_Core
 * @subpackage      Helpers
 *
 * @category        helpers
 *
 * @copyright       http://creativecommons.org/licenses/by/2.0/ - Please keep this comment intact
 * @author          gabe@fijiwebdesign.com
 * @contributers    adnan@barakatdesigns.net
 *
 * @version 0.1a
 */
class SMTP_validateEmail
{
    /**
     * PHP Socket resource to remote MTA
     *
     * @var resource
     */
    public $sock;

    /**
     * Current User being validated
     *
     * @var object
     */
    public $user;

    /**
     * Current domain where user is being validated
     *
     * @var string
     */
    public $domain;

    /**
     * List of domains to validate users on
     *
     * @var array
     */
    public $domains;

    /**
     * SMTP Port
     *
     * @var integer
     */
    public $port = 25;

    /**
     * Maximum Connection Time to an MTA
     *
     * @var integer
     */
    public $max_conn_time = 30;

    /**
     * Maximum time to read from socket
     *
     * @var integer
     */
    public $max_read_time = 5;

    /**
     * Username of sender
     *
     * @var string
     */
    public $from_user = 'user';

    /**
     * Host Name of sender
     *
     * @var string
     */
    public $from_domain = 'localhost';

    /**
     * Nameservers to use when make DNS query for MX entries
     *
     * @var array
     */
    public $nameservers = array('192.168.0.1');

    /**
     * Use debugging
     *
     * @return boolean
     */
    public $debug = true;

    /**
     * Initializes the Class
     *
     * @param Array[optional]  $emails List of Emails to Validate
     * @param String[optional] $sender Email of validator
     *
     * @return SMTP_validateEmail Instance
     * @return void
     */
    public function __construct($emails = false, $sender = false)
    {
        if ($emails) {
            $this->setEmails($emails);
        }
        if ($sender) {
            $this->setSenderEmail($sender);
        }
    }

    /**
     * Parse email address
     *
     * @param string $email email address
     *
     * @return array
     */
    public function _parseEmail($email)
    {
        $parts = explode('@', $email);
        $domain = array_pop($parts);
        $user = implode('@', $parts);

        return array($user, $domain);
    }

    /**
     * Set the Emails to validate
     *
     * @param array $emails list of Emails
     *
     * @return void
     */
    public function setEmails($emails)
    {
        foreach ($emails as $email) {
            list($user, $domain) = $this->_parseEmail($email);
            if (!isset($this->domains[$domain])) {
                $this->domains[$domain] = array();
            }
            $this->domains[$domain][] = $user;
        }
    }

    /**
     * Set the Email of the sender/validator
     *
     * @param string $email email address
     *
     * @return void
     */
    public function setSenderEmail($email)
    {
        $parts = $this->_parseEmail($email);
        $this->from_user = $parts[0];
        $this->from_domain = $parts[1];
    }

    /**
     * Validate Email Addresses
     *
     * Return associative List of Emails and their validation results
     *
     * @param string $emails Emails to validate (recipient emails)
     * @param string $sender Sender's Email
     *
     * @return array
     */
    public function validate($emails = false, $sender = false)
    {
        $results = array();
        if (!function_exists('getmxrr')) {
            return array($emails[0] => true);
        }

        if ($emails) {
            $this->setEmails($emails);
        }
        if ($sender) {
            $this->setSenderEmail($sender);
        }

        // query the MTAs on each Domain
        foreach ($this->domains as $domain => $users) {
            $mxs = array();

            // retrieve SMTP Server via MX query on domain
            list($hosts, $mxweights) = $this->queryMX($domain);

            // retrieve MX priorities
            for ($n = 0; $n < count($hosts); ++$n) {
                $mxs[$hosts[$n]] = $mxweights[$n];
            }
            asort($mxs);

            // last fallback is the original domain
            array_push($mxs, $this->domain);

            $this->debug(print_r($mxs, 1));

            $timeout = $this->max_conn_time / count($hosts);

            // try each host
            while (list($host) = each($mxs)) {
                // connect to SMTP server
                $this->debug("try $host:$this->port\n");
                if ($this->sock = fsockopen($host, $this->port, $errno, $errstr, (float) $timeout)) {
                    stream_set_timeout($this->sock, $this->max_read_time);
                    break;
                }
            }

            // did we get a TCP socket
            if ($this->sock) {
                $reply = fread($this->sock, 2082);
                $this->debug("<<<\n$reply");

                preg_match('/^([0-9]{3}) /ims', $reply, $matches);
                $code = isset($matches[1]) ? $matches[1] : '';

                if ($code != '220') {
                    // MTA gave an error...
                    foreach ($users as $user) {
                        $results[$user . '@' . $domain] = false;
                    }
                    continue;
                }

                // say helo
                $this->send("HELO " . $this->from_domain);

                // tell of sender
                $this->send("MAIL FROM: <" . $this->from_user . '@' . $this->from_domain . ">");

                // ask for each recepient on this domain
                foreach ($users as $user) {

                    // ask of recepient
                    $reply = $this->send("RCPT TO: <" . $user . '@' . $domain . ">");

                    // get code and msg from response
                    preg_match('/^([0-9]{3}) /ims', $reply, $matches);
                    $code = isset($matches[1]) ? $matches[1] : '';

                    if ($code == '250') {
                        // you received 250 so the email address was accepted
                        $results[$user . '@' . $domain] = true;
                    } elseif ($code == '451' || $code == '452') {
                        // you received 451 so the email address was greylisted (or some temporary error occured on the MTA) - so assume is ok
                        $results[$user . '@' . $domain] = true;
                    } else {
                        $results[$user . '@' . $domain] = false;
                    }
                }

                // quit
                $this->send("quit");

                // close socket
                fclose($this->sock);
            }
        }

        return $results;
    }

    /**
     * Send message
     *
     * @param string $msg message text
     *
     * @return string
     */
    public function send($msg)
    {
        fwrite($this->sock, $msg . "\r\n");

        $reply = fread($this->sock, 2082);

        $this->debug(">>>\n$msg\n");
        $this->debug("<<<\n$reply");

        return $reply;
    }

    /**
     * Query DNS server for MX entries
     *
     * @param string $domain domain value
     *
     * @return array
     */
    public function queryMX($domain)
    {
        $hosts = array();
        $mxweights = array();
        if (function_exists('getmxrr')) {
            getmxrr($domain, $hosts, $mxweights);
        } else {
            $hosts[] = "TRUE";
            $mxweights[] = "TRUE";
        }

        return array($hosts, $mxweights);
    }

    /**
     * Simple function to replicate PHP 5 behaviour. http://php.net/microtime
     *
     * @return float
     */
    public function microtime_float()
    {
        list($usec, $sec) = explode(" ", microtime());

        return ((float) $usec + (float) $sec);
    }

    /**
     * Display debugging message
     *
     * @param string $str debugging message
     *
     * @return void
     */
    public function debug($str)
    {
        if ($this->debug) {
            echo htmlentities($str);
        }
    }
}
