<?php

namespace Pg\Modules\Currencies\Models;

/**
 * Currencies module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

define('PAYMENTS_CURRENCY_TABLE', DB_PREFIX . 'payments_currency');

/**
 * Currencies model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Currencies
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Currencies_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Currency object properties in data source
     *
     * @var array
     */
    protected $fields = array(
        'id',
        'gid',
        'abbr',
        'name',
        'per_base',
        'template',
        'is_default',
        'status',
    );

    /**
     * Currency cache
     *
     * @param array
     */
    protected $currency_cache = array();

    /**
     * Currency by default
     *
     * @var array
     */
    public $default_currency = array();

    /**
     * Basic currency
     *
     * @var array
     */
    public $base_currency = array();

    /**
     * Caurrency's objects
     *
     * @var array
     */
    public $currencies = array();

    /**
     * Provider services of currency rates
     *
     * @var array
     */
    public $rates_updaters = array("yahoo"/*, "xe"*/);

    /**
     * Constructor
     *
     * @return Currencies_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->default_currency = $this->get_currency_default();
    }

    /**
     * Return currency object by GUID
     *
     * @param string $gid currency guid
     *
     * @return array
     */
    public function get_currency_by_gid($gid)
    {
        if (empty($this->currency_cache[$gid])) {
            $result = $this->ci->db->select(implode(", ", $this->fields))->from(PAYMENTS_CURRENCY_TABLE)->where("gid", $gid)->get()->result_array();
            if (!empty($result)) {
                $result[0]['is_default'] = 0;
                $this->currency_cache[$gid] = $result[0];
            } else {
                $this->currency_cache[$gid] = array();
            }
        }

        return $this->currency_cache[$gid];
    }

    /**
     * Return currency object by identifier
     *
     * @param integer $id currency identifier
     *
     * @return array
     */
    public function get_currency_by_id($id)
    {
        $result = $this->ci->db->select(implode(",", $this->fields))->from(PAYMENTS_CURRENCY_TABLE)->where("id", $id)->get()->result_array();
        if (!empty($result)) {
            $result[0]['is_default'] = 0;
            $r = $result[0];
            if (empty($this->currency_cache[$r["gid"]])) {
                $this->currency_cache[$r["gid"]] = $r;
            } else {
                $r = $this->currency_cache[$r["gid"]];
            }
        } else {
            $r = array();
        }

        return $r;
    }

    /**
     * Return currency object by default
     *
     * @param boolean $base get basic
     *
     * @return array
     */
    public function get_currency_default($base = false)
    {
        $currency_id = $this->ci->session->userdata("currency_id");
        $currency_id = intval($currency_id);

        $this->ci->db->select(implode(", ", $this->fields));
        $this->ci->db->from(PAYMENTS_CURRENCY_TABLE);

        if ($currency_id && !$base) {
            $this->ci->db->where("id", $currency_id);
        } else {
            $this->ci->db->where("is_default", '1');
        }

        $result = $this->ci->db->get()->result_array();
        if (!empty($result)) {
            $r = $result[0];
            if ($base) {
                return $r;
            } else {
                $r['is_default'] = 1;
            }
            $this->currency_cache[$r["gid"]] = $r;
        } else {
            $r = array();
        }

        return $r;
    }

    /**
     * Return currencies objects as array
     *
     * @param array $params            sql criteria of query to data source
     * @param array $filter_object_ids filters identifiers
     * @param array $order_by          sorting data
     *
     * @return array
     */
    public function get_currency_list($params = array(), $filter_object_ids = null, $order_by = null)
    {
        $this->ci->db->select(implode(", ", $this->fields));
        $this->ci->db->from(PAYMENTS_CURRENCY_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $r['is_default'] = 0;
                if (empty($this->currency_cache[$r["gid"]])) {
                    $this->currency_cache[$r["gid"]] = $r;
                } else {
                    $r = $this->currency_cache[$r["gid"]];
                }
                $data[] = $r;
            }

            return $data;
        }

        return array();
    }

    /**
     * Return currencies objects as associated array by identifiers
     *
     * @param array $params            sql criteria of query to data source
     * @param array $filter_object_ids filters identifiers
     * @param array $order_by          sorting data
     *
     * @return array
     */
    public function get_currency_list_by_key($params = array(), $filter_object_ids = null, $order_by = null)
    {
        $list = $this->get_currency_list($params, $filter_object_ids, $order_by);
        if (!empty($list)) {
            foreach ($list as $l) {
                $this->currencies[$l['id']] = $this->currencies[$l['gid']] = $data[$l['gid']] = $l;
            }

            return $data;
        } else {
            return array();
        }
    }

    /**
     * Return number of currencies objects
     *
     * @param array $params            sql criteria of query to data source
     * @param array $filter_object_ids filters identifiers
     *
     * @return integer
     */
    public function get_currency_count($params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select("COUNT(*) AS cnt");
        $this->ci->db->from(PAYMENTS_CURRENCY_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Validate currency object for saving to data source
     *
     * @param integer $id   currency identifier
     * @param array   $data currency data
     *
     * @return array
     */
    public function validate_currency($id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["gid"])) {
            $data["gid"] = strip_tags($data["gid"]);
            $data["gid"] = preg_replace("/[^a-z0-9]+/i", '', $data["gid"]);

            $return["data"]["gid"] = $data["gid"];

            if (empty($return["data"]["gid"])) {
                $return["errors"][] = l('error_currency_code_incorrect', 'currencies');
            }

            $param["where"]["gid"] = $return["data"]["gid"];
            if (!empty($id)) {
                $param["where"]["id <>"] = $id;
            }
            $gid_count = $this->get_currency_count($param);
            if ($gid_count > 0) {
                $return["errors"][] = l('error_currency_code_already_exists', 'currencies');
            }
        }

        if (isset($data["abbr"])) {
            $return["data"]["abbr"] = strip_tags($data["abbr"]);
            if (empty($return["data"]["abbr"])) {
                $return["errors"][] = l('error_abbr_required', 'currencies');
            }
        }

        if (isset($data["template"])) {
            $return["data"]["template"] = $data["template"];
        }

        if (isset($data["name"])) {
            $return["data"]["name"] = $data["name"];
            if (empty($return["data"]["name"])) {
                $return["errors"][] = l('error_currency_name_required', 'currencies');
            }
        }

        if (isset($data["per_base"])) {
            $return["data"]["per_base"] = (float) $data["per_base"];
        }

        if (isset($data["is_default"])) {
            $return["data"]["is_default"] = intval($data["is_default"]);
        }

        return $return;
    }

    /**
     * Save currency object to data source
     *
     * @param integer $id   currency identifier
     * @param array   $data currency data
     *
     * @return integer
     */
    public function save_currency($id, $data)
    {
        if (is_null($id)) {
            $this->ci->db->insert(PAYMENTS_CURRENCY_TABLE, $data);
            $id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $id);
            $this->ci->db->update(PAYMENTS_CURRENCY_TABLE, $data);
        }
        if (!empty($data["gid"])) {
            unset($this->currency_cache[$data["gid"]]);
        }

        return $id;
    }

    /**
     * Set currency object as default
     *
     * @param integer $id currency identifier
     *
     * @return void
     */
    public function set_default($id)
    {
        $data["is_default"] = '0';
        $this->ci->db->update(PAYMENTS_CURRENCY_TABLE, $data);

        $data["is_default"] = '1';
        $this->ci->db->where('id', $id);
        $this->ci->db->update(PAYMENTS_CURRENCY_TABLE, $data);

        return;
    }

    /**
     * Remove currency object by identifier
     *
     * @param integer $id currency identifier
     *
     * @return void
     */
    public function delete_currency($id)
    {
        $this->ci->db->where("id", $id);
        $this->ci->db->delete(PAYMENTS_CURRENCY_TABLE);

        return;
    }

    /**
     * Remove currency object by GUID
     *
     * @param string $gid currency GUID
     *
     * @return void
     */
    public function delete_currency_by_gid($gid)
    {
        $this->ci->db->where("gid", $gid);
        $this->ci->db->delete(PAYMENTS_CURRENCY_TABLE);
        unset($this->currency_cache[$gid]);

        return;
    }

    /**
     * Return amount value converted to base currency
     *
     * @param float  $value        amount value
     * @param string $currency_gid currency GUID
     * @param float
     */
    public function get_value_into_base_currency($value, $currency_gid)
    {
        $base_currency = $this->pg_module->get_module_config("start", "base_currency");
        if ($currency_gid == $base_currency) {
            return $value;
        }

        $data = $this->get_currency_by_gid($currency_gid);
        if (!empty($data)) {
            return $value * $data["per_base"];
        } else {
            return 0;
        }
    }

    /**
     * Update currencies rates
     *
     * @param string $rates_update_driver privider service of currency rates
     *
     * @return array
     */
    public function update_currency_rates($rates_update_driver)
    {
        $return = array("errors" => array(), "data" => array());

        $base_currency = $this->ci->pg_module->get_module_config("start", "base_currency");

        if (!$rates_update_driver || !in_array($rates_update_driver, $this->rates_updaters)) {
            $return["errors"][] = l("error_incorrect_rates_driver", "currencies");

            return $return;
        }

        ob_start();
        phpinfo(INFO_MODULES);
        $contents = ob_get_contents();
        ob_end_clean();
        $use_curl = strpos($contents, "curl") !== false;

        $currencies = $this->get_currency_list();
        if (empty($currencies)) {
            $return["errors"][] = l("error_empty_currencies", "currencies");

            return $return;
        }

        $model_name = $rates_update_driver . "_currency_rates_model";

        try {
            $this->ci->load->model("currencies/models/" . $model_name, $model_name);
            $results = $this->ci->{$model_name}->update_rates($base_currency, $currencies, $use_curl);
        } catch (\Exception $e) {
            $return["errors"][] = $e->getMessage();

            return $return;
        }

        foreach ($results as $currency_gid => $per_base) {
            $data = $this->get_currency_by_gid($currency_gid);
            if (empty($data)) {
                continue;
            }
            $validate_data = $this->validate_currency($data["id"], array("per_base" => $per_base));
            if (!empty($validate_data["errors"])) {
                continue;
            } else {
                $this->save_currency($data["id"], $validate_data["data"]);
            }
        }

        return $return;
    }

    /**
     * Update currencies rates automatically
     *
     * @return void
     */
    public function cron_update_currency_rates()
    {
        $use_rates_update = $this->pg_module->get_module_config("currencies", "use_rates_update");
        if (!$use_rates_update) {
            exit;
        }

        $rates_update_driver = $this->pg_module->get_module_config("currencies", "rates_update_driver");
        $this->update_currency_rates($rates_update_driver);
    }

    /**
     * Return currencies objects
     *
     * @return array
     */
    public function return_currencies()
    {
        if (!isset($this->currencies) || empty($this->currencies)) {
            $this->get_currency_list_by_key();
        }

        return $this->currencies;
    }

    /**
     * Activate/deactivate currency object
     *
     * Available status values: 1 - activate, 0 - deactivate
     *
     * @param integer $currency_id currency identifier
     * @param boolean $status      currency status
     *
     * @return boolean
     */
    public function activate_currency($currency_id, $status)
    {
        $data['status'] = $status ? 1 : 0;
        $currency = $this->get_currency_by_id($currency_id);
        if ($currency['status'] == $data['status'] || $currency['is_default']) {
            return false;
        }
        $this->save_currency($currency_id, $data);

        return true;
    }

}
