<?php

namespace Pg\Modules\Geomap\Models;

/**
 * Geomap module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

define('GEOMAPS_SETTINGS_TABLE', DB_PREFIX . 'geomap_settings');

/**
 * Geomap settings model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Geomap
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Geomap_settings_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Properties of geomap settings in data source
     *
     * @var array
     */
    protected $attrs = array(
        'id',
        'map_gid',
        'id_user',
        'id_object',
        'gid',
        'lat',
        'lon',
        'zoom',
        'view_type',
        'view_settings',
    );

    /**
     * Settings values by default
     *
     * @var array
     */
    protected $default_settings = array(
        "lat" => 0,
        "lon" => 0,
        "zoom" => 13,
        "view_type" => 1,
        "view_settings" => array(),
    );

    /**
     * Module name (GUID)
     *
     * @var string
     */
    public $module_gid = 'geomap';

    /**
     * Upload image name of map marker (GUID)
     *
     * @var string
     */
    public $upload_config_id = "map-marker-icon";

    /**
     * Constructor
     *
     * @return Geomap_settings_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return all settings of map object
     *
     * @param string  $map_gid   driver GUID
     * @param integer $id_user   user identifier
     * @param integer $id_object object identifier
     * @param string  $gid       map object GUID
     *
     * @return array/false
     */
    public function get_settings($map_gid, $id_user, $id_object, $gid)
    {
        $this->ci->db->select(implode(", ", $this->attrs));
        $this->ci->db->from(GEOMAPS_SETTINGS_TABLE);
        $this->ci->db->where("map_gid", $map_gid);
        $this->ci->db->where("id_user", $id_user);
        $this->ci->db->where("id_object", $id_object);
        $this->ci->db->where("gid", $gid);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $this->format_settings($results[0]);
        } else {
            return false;
        }
    }

    /**
     * Return inherited settings of map object
     *
     * @param string  $map_gid   driver GUID
     * @param integer $id_user   user identifier
     * @param integer $id_object object identifier
     * @param string  $gid       map object GUID
     *
     * @return array
     */
    public function get_parsed_settings($map_gid, $id_user = 0, $id_object = 0, $gid = "")
    {
        $settings = array('lat' => 0, 'lon' => 0, 'zoom' => 0, 'view_type' => 0, 'view_settings' => array());
        $this->ci->db->select(implode(", ", $this->attrs))
            ->from(GEOMAPS_SETTINGS_TABLE)
            ->where("map_gid", $map_gid);
        if (!empty($id_user)) {
            $this->ci->db->where("(id_user='" . intval($id_user) . "' OR id_user='0')");
        } else {
            $this->ci->db->where("id_user", 0);
        }
        if (!empty($id_object)) {
            $this->ci->db->where("(id_object='" . intval($id_object) . "' OR id_object='0')");
        } else {
            $this->ci->db->where("id_object", 0);
        }
        if (!empty($gid)) {
            $this->ci->db->where("(gid='" . $gid . "' OR gid='')");
        } else {
            $this->ci->db->where("gid", "");
        }
        $this->ci->db->order_by("id_object ASC")->order_by("id_user ASC")->order_by("gid ASC");
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $result) {
                $result = $this->format_settings($result);

                if (!empty($result['rent_marker_color'])) {
                    $settings['rent_marker_color'] = $result['rent_marker_color'];
                }

                if (!empty($result['sale_marker_color'])) {
                    $settings['sale_marker_color'] = $result['sale_marker_color'];
                }

                if (!empty($result['sold_marker_color'])) {
                    $settings['sold_marker_color'] = $result['sold_marker_color'];
                }

                if ($result['lat'] != 0 || $result['lon'] != 0) {
                    $settings['lat'] = $result['lat'];
                    $settings['lon'] = $result['lon'];
                }
                if ($result['zoom'] != 0) {
                    $settings['zoom'] = $result['zoom'];
                }
                if ($result['view_type'] != 0) {
                    $settings['view_type'] = $result['view_type'];
                }

                foreach ($this->attrs as $attr) {
                    if ($attr != 'view_settings') {
                        unset($result[$attr]);
                    }
                }

                $settings = array_merge($settings, $result);
            }
        } else {
            $settings = $this->default_settings;
        }

        return $settings;
    }

    /**
     * Save settings of map object to data source
     *
     * @param string  $map_gid   driver GUID
     * @param integer $id_user   user identifier
     * @param integer $id_object object identifier
     * @param string  $gid       map object gid
     * @param array   $data      map object data
     *
     * @return void
     */
    public function save_settings($map_gid, $id_user, $id_object, $gid, $data = array())
    {
        if ($this->is_settings_exists($map_gid, $id_user, $id_object, $gid)) {
            $this->ci->db->where('map_gid', $map_gid);
            $this->ci->db->where('id_user', $id_user);
            $this->ci->db->where('id_object', $id_object);
            $this->ci->db->where('gid', $gid);
            $this->ci->db->update(GEOMAPS_SETTINGS_TABLE, $data);
        } else {
            $data["map_gid"] = $map_gid;
            $data["id_user"] = $id_user;
            $data["id_object"] = $id_object;
            $data["gid"] = $gid;
            $this->ci->db->insert(GEOMAPS_SETTINGS_TABLE, $data);
        }
        $this->ci->load->model('Dynamic_blocks_model');
        $this->ci->Dynamic_blocks_model->cache_all_clear();
        return;
    }

    /**
     * Save icon of map marker
     *
     * @param string $map_gid   map driver GUID
     * @param string $gid       page object GUID
     * @param string $file_name upload file name
     *
     * @return void
     */
    public function save_marker_icon($map_gid, $gid, $file_name)
    {
        if (empty($file_name) || !isset($_FILES[$file_name]) ||
            !is_array($_FILES[$file_name]) || !is_uploaded_file($_FILES[$file_name]["tmp_name"])) {
            return;
        }

        $this->ci->load->model("Uploads_model");
        $img_return = $this->ci->Uploads_model->upload($this->upload_config_id, $map_gid . ($gid ? '_' . $gid : '') . "/", $file_name);

        if (!empty($img_return["error"])) {
            return;
        }

        $settings = $this->get_parsed_settings($map_gid, 0, 0, $gid);
        $settings['marker_icon'] = $img_return['file'];
        unset($settings['id']);
        $validate_data = $this->ci->Geomap_settings_model->validate_settings($settings);
        $this->save_settings($map_gid, 0, 0, $gid, $validate_data['data']);
    }

    /**
     * Remove settings of map object from data source
     *
     * @param string  $map_gid   map driver GUID
     * @param integer $id_user   user identifier
     * @param integer $id_object object identifier
     * @param string  $gid       map object GUID
     *
     * @return void
     */
    public function delete_settings($map_gid, $id_user, $id_object, $gid)
    {
        $this->ci->db->where('map_gid', $map_gid);
        $this->ci->db->where('id_user', $id_user);
        $this->ci->db->where('id_object', $id_object);
        $this->ci->db->where('gid', $gid);
        $this->ci->db->delete(GEOMAPS_SETTINGS_TABLE);

        return;
    }

    /**
     * Remove map marker icon
     *
     * @param string $map_gid map driver GUID
     * @param string $gid     page object GUID
     *
     * @return void
     */
    public function delete_marker_icon($map_gid, $gid)
    {
        $this->ci->load->model("Uploads_model");
        $settings = $this->get_parsed_settings($map_gid, 0, 0, $gid);
        $this->ci->Uploads_model->delete_upload($this->upload_config_id, $map_gid . ($gid ? '_' . $gid : '') . "/", $settings["marker_icon"]);
        unset($settings['marker_icon']);
        unset($settings['id']);

        $validate_data = $this->ci->Geomap_settings_model->validate_settings($settings);
        $this->save_settings($map_gid, 0, 0, $gid, $validate_data['data']);
    }

    /**
     * Validate map object settings for saving to data source
     *
     * @param array $data settings data
     *
     * @return array
     */
    public function validate_settings($data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["rent_marker_color"])) {
            $data["rent_marker_color"] = trim(strip_tags($data["rent_marker_color"]));
        }

        if (isset($data["markers_show_square"])) {
            $data["markers_show_square"] = intval($data["markers_show_square"]);
        }

        if (isset($data["use_clasterer"])) {
            $data["use_clasterer"] = intval($data["use_clasterer"]);
        }

        if (isset($data["use_geotargetting"])) {
            $data["use_geotargetting"] = intval($data["use_geotargetting"]);
        }

        if (isset($data["sale_marker_color"])) {
            $data["sale_marker_color"] = trim(strip_tags($data["sale_marker_color"]));
        }

        if (isset($data["sold_marker_color"])) {
            $data["sold_marker_color"] = trim(strip_tags($data["sold_marker_color"]));
        }

        if (isset($data["lat"])) {
            $return["data"]["lat"] = strval(floatval($data["lat"]));
            unset($data["lat"]);
        }

        if (isset($data["lon"])) {
            $return["data"]["lon"] = strval(floatval($data["lon"]));
            unset($data["lon"]);
        }

        if (isset($data["zoom"])) {
            $return["data"]["zoom"] = intval($data["zoom"]);
            unset($data["zoom"]);
        }

        if (isset($data["view_type"])) {
            $return["data"]["view_type"] = intval($data["view_type"]);
            unset($data["view_type"]);
        }

        if (isset($data["control_position"])) {
            $temp = array();
            foreach ($data["control_position"] as $key => $value) {
                $temp[$value] = true;
            }
            $data["control_position"] = $temp;   
        }

        $return["data"]["view_settings"] = serialize($data);
        return $return;
    }

    /**
     * Validate upload icon of map marker
     *
     * @param string $file_name upload file name
     *
     * @return array
     */
    public function validate_marker_icon($file_name)
    {
        $return = array('errors' => array(), 'data' => array());
        if (isset($_FILES[$file_name]) && is_array($_FILES[$file_name]) && is_uploaded_file($_FILES[$file_name]["tmp_name"])) {
            $this->ci->load->model("Uploads_model");
            $validate_image = $this->ci->Uploads_model->validate_upload($this->upload_config_id, $file_name);
            if ($validate_image["error"]) {
                $return["errors"] = $validate_image["error"];
            }
        }

        return $return;
    }

    /**
     * Format settings data of map object
     *
     * @param array $data settings data
     *
     * @return array
     */
    public function format_settings($data)
    {
        $view_settings = $data["view_settings"] ? (array) unserialize($data["view_settings"]) : array();
        unset($data["view_settings"]);
        foreach ($view_settings as $name => $value) {
            $data[$name] = $value;
        }
        if (isset($data['marker_icon']) && !empty($data["marker_icon"])) {
            $this->ci->load->model('Uploads_model');
            $data["media"]["icon"] = $this->ci->Uploads_model->format_upload($this->upload_config_id, $data['map_gid'] . ($data['gid'] ? '_' . $data['gid'] : ''), $data["marker_icon"]);
        }

        return $data;
    }

    /**
     * Check exists settings of map object
     *
     * @param string  $map_gid   driver GUID
     * @param integer $id_user   user identifier
     * @param integer $id_object object identifier
     * @param string  $gid       map object GUID
     *
     * @return boolean
     */
    public function is_settings_exists($map_gid, $id_user, $id_object, $gid)
    {
        $this->ci->db->select("COUNT(*) AS cnt");
        $this->ci->db->from(GEOMAPS_SETTINGS_TABLE);
        $this->ci->db->where('map_gid', $map_gid);
        $this->ci->db->where('id_user', $id_user);
        $this->ci->db->where('id_object', $id_object);
        $this->ci->db->where('gid', $gid);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results) && intval($results[0]["cnt"])) {
            return true;
        }

        return false;
    }

    /**
     * Return available maps
     *
     * @param string $map_gid map guid
     *
     * @return array
     */
    public function get_maps_lists($map_gid)
    {
        $this->ci->db->select("gid");
        $this->ci->db->from(GEOMAPS_SETTINGS_TABLE);
        $this->ci->db->where('map_gid', $map_gid);
        $this->ci->db->where('id_user', 0);
        $this->ci->db->where('id_object', 0);
        $this->ci->db->where('gid !=', '');
        $results = $this->ci->db->get()->result_array();

        return $results;
    }

    /**
     * Import languages of geomap module
     *
     * @param array $data       map data
     * @param array $langs_file data by languages
     * @param array $langs_ids  languages identifiers
     *
     * @return void
     */
    public function update_lang($data, $langs_file, $langs_ids)
    {
        foreach ($data as $value) {
            $this->ci->pg_language->pages->set_string_langs(
                "geomap", $value, $langs_file[$value], $langs_ids);
        }
    }

    /**
     * Export languages of geomap module
     *
     * @param array $data      map data
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function export_lang($data, $langs_ids)
    {
        $langs = array();

        return array_merge($langs, $this->ci->pg_language->export_langs("geomap", (array) $data, $langs_ids));
    }

}
