<?php

/**
 * Import module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

define("IMPORT_TABLE", DB_PREFIX . "import");
define("IMPORT_DATA_TABLE", DB_PREFIX . "import_data");
define("IMPORT_LOG_TABLE", DB_PREFIX . "import_log");

/**
 * Import model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Import
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Import_model extends Model
{
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $CI;

    /**
     * Link to database object
     *
     * @var object
     */
    protected $DB;

    /**
     * Properties of import object in data source
     *
     * @var array
     */
    protected $_fields = array(
        "id",
        "gid_driver",
        "id_object",
        "settings",
        "relations",
        "search_data",
        "input_type",
        "filename",
        "scheduler",
        "labels",
        "position",
        "total",
        "processed",
        "imported",
        "failed",
        "status",
        "date_created",
    );

    /**
     * Settings for formatting selection object
     *
     * @var array
     */
    protected $format_settings = array(
        "use_format"  => true,
        "get_driver"  => true,
        "get_module"  => true,
    );

    /**
     * Drivers cache
     *
     * @var array
     */
    protected $driver_cache = array();

    /**
     * Modules cache
     *
     * @var array
     */
    protected $module_cache = array();

    /**
     * Path to uploaded files
     *
     * @var string
     */
    protected $base_path = "import/";

    /**
     * Import data limit per request
     *
     * @var integer
     */
    protected $import_limit = 100;

    /**
     * Constructor
     *
     * @return Import_model
     * @required Import_driver_model
     * @required Import_module_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
        $this->DB = &$this->CI->db;
        $this->CI->load->model("import/models/Import_driver_model");
        $this->CI->load->model("import/models/Import_module_model");
    }

    /**
     * Return import driver data by GUID
     *
     * @param string $driver_gid driver guid
     *
     * @return array
     */
    public function get_driver_by_gid($driver_gid)
    {
        if (!isset($this->driver_cache[$driver_gid])) {
            $driver_data = $this->CI->Import_driver_model->get_driver_by_gid($driver_gid);
            if (!is_array($driver_data) || !count($driver_data)) {
                return false;
            }
            $this->driver_cache[$driver_data["id"]] = $driver_data;
            $this->driver_cache[$driver_gid] = $driver_data;
        }

        if (is_array($this->driver_cache[$driver_gid]) && count($this->driver_cache[$driver_gid])) {
            return $this->driver_cache[$driver_gid];
        } else {
            return false;
        }
    }

    /**
     * Return import object by identifier
     *
     * @param integer $import_id import data identifier
     * @param boolean $formatted format results
     *
     * @return array
     */
    public function get_data_by_id($import_id, $formatted = false)
    {
        $import_id = intval($import_id);

        $this->DB->select(implode(", ", $this->_fields));
        $this->DB->from(IMPORT_TABLE);
        $this->DB->where("id", $import_id);

        //_compile_select;
        $result = $this->DB->get()->result_array();
        if (!empty($result)) {
            $rt = array($result[0]);
            if ($formatted) {
                $rt = $this->format_data($rt);
            }

            return $rt[0];
        } else {
            return false;
        }
    }

    /**
     * Save import object to data source
     *
     * @param integer $import_id import data identifier
     * @param array   $data      import data
     *
     * @return integer
     */
    public function save_data($import_id, $data)
    {
        if (!$import_id) {
            $data["date_created"] = date("Y-m-d H:i:s");
            $this->DB->insert(IMPORT_TABLE, $data);
            $import_id = $this->DB->insert_id();
        } else {
            $this->DB->where("id", $import_id);
            $this->DB->update(IMPORT_TABLE, $data);
        }

        return $import_id;
    }

    /**
     * Remove import object from data source by identifier
     *
     * @param integer $import_id data identifier
     *
     * @return void
     */
    public function delete_data($import_id)
    {
        $this->DB->where("id_import", $import_id);
        $this->DB->delete(IMPORT_DATA_TABLE);

        $this->DB->where("id", $import_id);
        $this->DB->delete(IMPORT_TABLE);
    }

    /**
     * Remove uploaded file
     *
     * @param integer $import_id data identifier
     * @param boolean $deep      remove all files
     *
     * @return void
     */
    public function delete_file($import_id, $deep = false)
    {
        $import_data = $this->get_data_by_id($import_id, true);
        if (!$import_data) {
            return false;
        }
        if ($deep) {
            if (!file_exists(UPLOADPATH . $this->base_path . $import_id)) {
                return;
            }
            foreach (scandir(UPLOADPATH . $this->base_path . $import_id) as $filename) {
                if ($filename == "." || $filename == "..") {
                    continue;
                }
                unlink(UPLOADPATH . $this->base_path . $import_id . "/" . $filename);
            }
            rmdir(UPLOADPATH . $this->base_path . $import_id);
            if (!$import_data["filename"]) {
                return true;
            }
        } else {
            if (!$import_data["filename"]) {
                return false;
            }
            unlink(UPLOADPATH . $this->base_path . $import_id . "/" . $import_data["filename"]);
        }
        $data["filename"] = "";
        $this->DB->where("id", $import_id);
        $this->DB->update(IMPORT_TABLE, $data);

        return true;
    }

    /**
     * Return modules filter
     *
     * @param integer $module_id module identifier
     *
     * @return array
     */
    protected function _get_data_by_module($module_id)
    {
        if (!$module_id) {
            return array();
        }
        $params["where"]["id_module"] = $module_id;

        return $params;
    }

    /**
     * Return drivers filter
     *
     * @param string $driver_gid driver guid
     *
     * @return array
     */
    protected function _get_data_by_driver($driver_gid)
    {
        if (!$driver_gid) {
            return array();
        }
        $params["where"]["gid_driver"] = $driver_gid;

        return $params;
    }

    /**
     * Return active filter
     *
     * @param string $active driver guid
     *
     * @return array
     */
    protected function _get_data_by_active($active)
    {
        if (!$active) {
            return array();
        }
        $params["where"]["status"] = $active;

        return $params;
    }

    /**
     * Return import objects as array
     *
     * @param integer $page     page of results
     * @param string  $limits   results per page
     * @param array   $order_by sorting data
     * @param array   $params   filters datas
     *
     * @return array
     */
    protected function _get_data_list($page = null, $limits = null, $order_by = null, $params = array())
    {
        $this->DB->select(implode(", ", $this->_fields));
        $this->DB->from(IMPORT_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->DB->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->DB->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->DB->where($value);
            }
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->_fields)) {
                    $this->DB->order_by($field . " " . $dir);
                }
            }
        } elseif ($order_by) {
            $this->DB->order_by($order_by);
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->DB->limit($limits, $limits * ($page - 1));
        }

        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $r;
            }

            return $this->format_data($data);
        }

        return array();
    }

    /**
     * Return number of import objects
     *
     * @param array $params filters data
     *
     * @return integer
     */
    protected function _get_data_count($params = array())
    {
        $this->DB->select("COUNT(*) AS cnt");
        $this->DB->from(IMPORT_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->DB->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->DB->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->DB->where($value);
            }
        }

        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Return import objects as array
     *
     * @param array   $filters  selection filters
     * @param integer $page     page of results
     * @param string  $limits   results per page
     * @param array   $order_by sorting data
     *
     * @return array
     */
    public function get_data_list($filters = array(), $page = null, $limits = null, $order_by = null)
    {
        $params = array();
        foreach ($filters as $filter => $value) {
            $params = array_merge_recursive($params, $this->{"_get_data_by_" . $filter}($value));
        }

        return $this->_get_data_list($page, $limits, $order_by);
    }

    /**
     * Return number of import objects
     *
     * @param array $filters selection filters
     *
     * @return integer
     */
    public function get_data_count($filters = array())
    {
        $params = array();
        foreach ($filters as $filter => $value) {
            $params = array_merge_recursive($params, $this->{"_get_data_by_" . $filter}($value));
        }

        return $this->_get_data_count($params);
    }

    /**
     * Validate import data
     *
     * @param integer $import_id import identifier
     * @param array   $data      import data
     *
     * @return array
     */
    public function validate_data($import_id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if ($import_id) {
            $this->set_format_settings('use_format', false);
            $import_data = $this->get_data_by_id($import_id);
            $this->set_format_settings('use_format', true);
        } else {
            $import_data = array('input_type' => '');
        }

        if (isset($data['id'])) {
            $return['data']['id'] = intval($data['id']);
            if (empty($return['data']['id'])) {
                unset($return['data']['id']);
            }
        }

        if (isset($data['gid_driver'])) {
            $return['data']['gid_driver'] = trim(strip_tags($data["gid_driver"]));
            if (empty($return['data']['gid_driver'])) {
                $return["errors"][] = l("error_empty_driver", "import");
            } else {
                $driver = $this->CI->Import_driver_model->get_driver_by_gid($return['data']['gid_driver']);
                if ($driver) {
                    $model_name = "import_" . $driver['gid'] . "_model";
                    $this->CI->load->model("import/models/drivers/" . $model_name, $model_name);
                    $validate_driver_data = $this->CI->{$model_name}->validate_data($data);
                    $return["errors"] = array_merge($return["errors"], $validate_driver_data["errors"]);
                } else {
                    $return["errors"][] = l("error_invalid_driver", "import");
                }
            }
        } elseif (!$import_id) {
            $return["errors"][] = l("error_empty_driver", "import");
        }

        if (isset($data['id_object'])) {
            $return['data']['id_object'] = intval($data['id_object']);
            if (empty($return['data']['id_object'])) {
                $return["errors"][] = l("error_empty_object", "import");
            } else {
                $module = $this->CI->Import_module_model->get_module_by_id($return['data']['id_object']);
                if (!$module) {
                    $return["errors"][] = l("error_invalid_module", "import");
                }
            }
        } elseif (!$import_id) {
            $return["errors"][] = l("error_empty_object", "import");
        }

        if (isset($data["settings"])) {
            if (!is_array($data["settings"])) {
                $data["settings"] = array();
            }
            $return["data"]["settings"] = serialize($data["settings"]);
        }

        if (isset($data["relations"])) {
            if (!is_array($data["relations"])) {
                $data["relations"] = array();
            }
            $return["data"]["relations"] = serialize($data["relations"]);
        }

        if (isset($data["search_data"])) {
            if (!is_array($data["search_data"])) {
                $data["search_data"] = array();
            }
            $return["data"]["search_data"] = serialize($data["search_data"]);
        }

        if (isset($data["labels"])) {
            if (!is_array($data["labels"])) {
                $data["labels"] = array();
            }
            $return["data"]["labels"] = serialize($data["labels"]);
        }

        if (isset($data['input_type'])) {
            $return['data']['input_type'] = $import_data['input_type'] = trim(strip_tags($data["input_type"]));
        }

        if (isset($data['filename'])) {
            $return['data']['filename'] = trim(strip_tags($data["filename"]));
        }

        if ($import_data["input_type"] == "integration") {
            if (isset($data["scheduler"])) {
                $return["data"]["scheduler"] = $data["scheduler"];
                if (empty($return["data"]["scheduler"])) {
                    $return["errors"][] = l("error_empty_scheduler", "import");
                } else {
                    switch ($return["data"]["scheduler"]["type"]) {
                        case "cron":
                            $return["data"]["scheduler"] = array(
                                "type"    => "cron",
                                "date"    => strval($return["data"]["scheduler"]["date"]),
                                "hours"   => intval($return["data"]["scheduler"]["hours"]),
                                "minutes" => intval($return["data"]["scheduler"]["minutes"]),
                                "period"  => strval($return["data"]["scheduler"]["period"]),
                            );
                            $return["data"]["scheduler"]["date_for_cron"] = strtotime($return["data"]["scheduler"]["date"] . " " . $return["data"]["scheduler"]["hours"] . ":" . $return["data"]["scheduler"]["minutes"]);
                            $return["data"]["scheduler"] = serialize($return["data"]["scheduler"]);
                        break;
                        case "intime":
                            $return["data"]["scheduler"] = array(
                                "type"    => "intime",
                                "date"    => strval($return["data"]["scheduler"]["date"]),
                                "hours"   => intval($return["data"]["scheduler"]["hours"]),
                                "minutes" => intval($return["data"]["scheduler"]["minutes"]),
                            );
                            $return["data"]["scheduler"]["date_for_cron"] = strtotime($return["data"]["scheduler"]["date"] . " " . $return["data"]["scheduler"]["hours"] . ":" . $return["data"]["scheduler"]["minutes"]);
                            $return["data"]["scheduler"] = serialize($return["data"]["scheduler"]);
                        break;
                        case "manual":
                            $return["data"]["scheduler"] = serialize(array("type" => "manual"));
                        break;
                        default:
                            unset($return["data"]["scheduler"]);
                        break;
                    }
                }
            }
        }

        if (isset($data['position'])) {
            $return['data']['position'] = intval($data["position"]);
        }

        if (isset($data['total'])) {
            $return['data']['total'] = intval($data["total"]);
        }

        if (isset($data['processed'])) {
            $return['data']['processed'] = intval($data["processed"]);
        }

        if (isset($data['imported'])) {
            $return['data']['imported'] = intval($data["imported"]);
        }

        if (isset($data['failed'])) {
            $return['data']['failed'] = intval($data["failed"]);
        }

        if (isset($data['status'])) {
            $return['data']['status'] = intval($data["status"]);
        }

        if (isset($data['date_created'])) {
            $value = strtotime($data['date_created']);
            if ($value > 0) {
                $return['data']['date_created'] = date("Y-m-d", $value);
            }
        }

        return $return;
    }

    /**
     * Set format settings
     *
     * @param string $name  parameter name
     * @param mixed  $value parameter value
     *
     * @return void
     */
    public function set_format_settings($name, $value = false)
    {
        if (!is_array($name)) {
            $name = array($name => $value);
        }
        foreach ($name as $key => $item) {
            $this->format_settings[$key] = $item;
        }
    }

    /**
     * Format import data
     *
     * @param array $data import data
     *
     * @return array
     */
    public function format_data($data)
    {
        if (!$this->format_settings["use_format"]) {
            return $data;
        }

        $drivers_data = $this->CI->Import_driver_model->get_drivers();
        foreach ($drivers_data as $driver_data) {
            $this->driver_cache[$driver_data["gid"]] = $driver_data;
        }

        $modules_data = $this->CI->Import_module_model->get_modules();
        foreach ($modules_data as $module_data) {
            $this->module_cache[$module_data["id"]] = $module_data;
        }

        $drivers_search = array();
        $modules_search = array();

        foreach ($data as $key => $item) {
            $data[$key] = $item;
            //get driver
            if ($this->format_settings["get_driver"]) {
                $drivers_search[] = $item["gid_driver"];
            }
            //get module
            if ($this->format_settings["get_module"]) {
                $modules_search[] = $item["id_object"];
            }

            if ($item["settings"]) {
                $data[$key]["settings"] = (array) unserialize($item["settings"]);
            } else {
                $data[$key]["settings"] = array();
            }

            if ($item["relations"]) {
                $data[$key]["relations"] = (array) unserialize($item["relations"]);
            } else {
                $data[$key]["relations"] = array();
            }

            if ($item["search_data"]) {
                $data[$key]["search_data"] = (array) unserialize($item["search_data"]);
            } else {
                $data[$key]["search_data"] = array();
            }

            if ($item["labels"]) {
                $data[$key]["labels"] = (array) unserialize($item["labels"]);
            } else {
                $data[$key]["labels"] = array();
            }

            if ($item["scheduler"]) {
                $data[$key]["scheduler"] = (array) unserialize($item["scheduler"]);
                if (!isset($data[$key]["scheduler"]["type"])) {
                    $data[$key]["scheduler"] = array("type" => "manual");
                }
            } else {
                $data[$key]["scheduler"] = array();
            }
        }

        if ($this->format_settings["get_driver"] && !empty($drivers_search)) {
            foreach ($data as $key => $item) {
                $data[$key]["driver"] = (isset($this->driver_cache[$item["gid_driver"]])) ?
                    $this->driver_cache[$item["gid_driver"]] : $this->CI->Import_driver_model->format_default_driver($item["gid_driver"]);
            }
        }

        if ($this->format_settings["get_module"] && !empty($modules_search)) {
            foreach ($data as $key => $item) {
                $data[$key]["module"] = (isset($this->module_cache[$item["id_object"]])) ?
                    $this->module_cache[$item["id_object"]] : $this->CI->Import_module_model->format_default_module($item["id_object"]);
            }
        }

        return $data;
    }

    /**
     * Validate import file
     *
     * @param string $upload_gid upload file guid
     * @param string $data       import data
     *
     * @return array
     */
    public function validate_upload($upload_gid, $data)
    {
        $return = array("errors" => array(), "data" => $data);

        $this->CI->load->helper("upload");

        $model_name = "import_" . $data["gid_driver"] . "_model";
        $this->CI->load->model("import/models/drivers/" . $model_name, $model_name);

        //// upload src file
        $upload_config = array(
            "allowed_types" => $this->CI->{$model_name}->allowed_types,
            "overwrite"     => true,
        );

        $result = validate_file($upload_gid, $upload_config);
        if ($result["error"]) {
            $return["errors"] = $result["error"];
        }

        return $return;
    }

    /**
     * Upload import file
     *
     * @param string  $upload_gid upload file guid
     * @param integer $import_id  import object identifier
     *
     * @return array
     */
    public function upload($upload_gid, $import_id)
    {
        $return = array("errors" => array(), "data" => array());

        $import_data = $this->get_data_by_id($import_id, true);
        $model_name = "import_" . $import_data["gid_driver"] . "_model";
        $this->CI->load->model("import/models/drivers/" . $model_name, $model_name);
        $this->CI->load->helper("upload");

        //// upload src file
        $upload_config = array(
            "allowed_types" => $this->CI->{$model_name}->allowed_types,
            "overwrite"     => true,
        );

        $result = upload_file($upload_gid, TEMPPATH . $this->base_path, $upload_config);
        if (!empty($result["error"])) {
            $return["errors"] = $result["error"];
        } else {
            @mkdir(UPLOADPATH . $this->base_path . $import_id);
            $pathinfo = pathinfo($result["data"]["full_path"]);
            if ($pathinfo["extension"] == "zip") {
                $this->CI->load->library("unzip");
                $this->CI->unzip->fileName = $result["data"]["full_path"];
                $this->CI->unzip->targetDir = UPLOADPATH . $this->base_path . $import_id;
                $this->CI->unzip->unzipAll();
                $this->CI->unzip->close();
                $result["data"]["file_name"] = substr($result["data"]["file_name"], 0, -4) . "." . $this->CI->{$model_name}->extension;
                if (!file_exists(UPLOADPATH . $this->base_path . $import_id . '/' . $result["data"]["file_name"])) {
                    $found = false;
                    foreach (scandir(UPLOADPATH . $this->base_path . $import_id) as $file) {
                        if ($file == '.' || $file == '..' || is_dir(UPLOADPATH . $this->base_path . $import_id . '/' . $file)) {
                            continue;
                        }
                        $pathinfo = pathinfo(UPLOADPATH . $this->base_path . $import_id . '/' . $file);
                        if ($pathinfo['extension'] == $this->CI->{$model_name}->extension) {
                            @copy(UPLOADPATH . $this->base_path . $import_id . '/' . $file, UPLOADPATH . $this->base_path . $import_id . '/' . $result["data"]["file_name"]);
                            @unlink(UPLOADPATH . $this->base_path . $import_id . '/' . $file);
                            $found = true;
                            break;
                        }
                    }
                    if (!$found) {
                        $return["errors"][] = l('error_empty_data', 'import');

                        return $return;
                    }
                }
            } else {
                @copy($result["data"]["full_path"], UPLOADPATH . $this->base_path . $import_id . "/" . $result["data"]["file_name"]);
            }

            $save_data = array('filename' => $result["data"]["file_name"]);
            $this->save_data($import_id, $save_data);

            $return["data"]["filename"] = $result["data"]["file_name"];
        }

        return $return;
    }

    /**
     * Parse uploaded data
     *
     * @param integer $import_id import object identifier
     *
     * @return array
     */
    public function parse_data($import_id)
    {
        @ini_set("max_execution_time", 0);

        $return = array("errors" => array(), "data" => array());

        $import_data = $this->get_data_by_id($import_id, true);
        if (!$import_data) {
            $return["errors"][] = l("error_empty_data", "import");

            return $return;
        }

        $model_name = "Import_" . $import_data["gid_driver"] . "_model";
        $this->CI->load->model("import/models/drivers/" . $model_name, $model_name);

        if ($import_data['processed'] || $import_data['imported'] || $import_data['failed']) {
            $this->CI->{$model_name}->set_depth();
        }

        $filename = UPLOADPATH . $this->base_path . $import_id . "/" . $import_data["filename"];
        $results = $this->CI->{$model_name}->parse(
            $filename,
            $import_data["settings"],
            $import_data['gid_driver'],
            $import_data["relations"],
            $import_data["search_data"],
            $import_data["position"]);

        //$this->decline_import_data($import_data["id"]);

        if ($import_data['gid_driver']['is_logging'] && !empty($results['system_ids'])) {
            $objects_ids = $this->get_object_ids($import_data['gid_driver'], $import_data['id_object'], $results['system_ids']);
            foreach ($results["data"] as $key => $item) {
                $results["data"][$key][$results['site_id']] = isset($objects_ids[$results['system_id']]) ?
                    $objects_ids[$results['system_id']] : 0;
            }
        }

        foreach ($results["data"] as $item) {
            $save_data = array(
                "id_import" => $import_id,
                "data"     => serialize($item),
                "status"   => 0,
            );
            $import_data_id = $this->save_import_data(null, $save_data);
            if ($import_data['gid_driver']['is_logging']) {
                $log_data = array(
                    'gid_driver'     => $import_data['gid_driver'],
                    'id_object'      => $import_data['id_object'],
                    'id_import_data' => $import_data_id,
                    'system_id'      => isset($item[$results['system_id']]) ? $item[$results['system_id']] : 0,
                );
                $this->save_log(null, $log_data);
            }
        }

        $results["processed"] += $import_data['processed'];

        $data["id"] = $import_data["id"];
        $data["position"] = $results["position"];
        $data["status"] = $results["status"];
        $data["processed"] = $results["processed"];
        $data["imported"] = $import_data["imported"];
        //if(!empty($results["total"])) $data["total"] = $import_data['total'] + $results["total"];
        if (!empty($results["settings"])) {
            $data["settings"] = serialize($results["settings"]);
        }
        $this->save_data($import_id, $data);

        $this->approve_import_data($import_data["id"]);

        if ($results["status"] == 1) {
            $this->delete_file($import_data["id"]);
        }

        $return["data"]["status"] = $results["status"];
        $return["data"]["processed"] = $results["processed"];

        return $return;
    }

    /**
     * Import prepared data
     *
     * @param integer $import_id      import identifier
     * @param integer $user_id        user identifier
     * @param boolean $use_moderation moderate data
     *
     * @return array
     */
    public function make_import_data($import_id, $user_id = 0, $use_moderation = false)
    {
        @ini_set("max_execution_time", 0);

        $import_data = $this->get_data_by_id($import_id, true);
        if (!$import_data) {
            return array("errors" => array(l("error_empty_data", "import")), "data" => array());
        }

        $return = array("errors" => array(), "data" => array());
        foreach ((array) $import_data["relations"] as $index => $relation) {
            if (!$relation["link"]) {
                unset($import_data["relations"][$index]);
            }
        }
        if (empty($import_data["relations"])) {
            $return["errors"][] = l("error_empty_relations", "import");
        }

        if (empty($user_id) && !empty($import_data['driver']['settings']['agent'])) {
            $user_id = $import_data['driver']['settings']['agent'];
        }

        //make unique relations
        $relations = array();
        foreach ($import_data["relations"] as $index => $relation) {
            if (array_key_exists($relation['link'], $relations)) {
                unset($import_data["relations"][$relations[$relation['link']]]);
            }
            $relations[$relation['link']] = $index;
        }

        $data  = $this->get_import_data($import_data["id"]);
        if (empty($data)) {
            $return["errors"][] = l("error_empty_data", "import");
        }

        if (empty($return["errors"])) {
            foreach ($data as $id => $row) {
                $object_data = array();
                $attachements = array();
                foreach ($import_data["relations"] as $index => $relation) {
                    switch ($relation["type"]) {
                        case "text":
                            $object_data[$relation["link"]] = strval($row[$index]);
                        break;
                        case "int":
                            $object_data[$relation["link"]] = intval($row[$index]);
                        break;
                        case "file":
                            if (!$row[$index]) {
                                break;
                            }
                            $attachements = explode("|", $row[$index]);
                            foreach ($attachements as $i => $attachement) {
                                $attachements[$i] = UPLOADPATH . $this->base_path . $import_id . "/" . $attachement;
                                if (!file_exists($attachements[$i])) {
                                    unset($attachements[$i]);
                                }
                            }
                            $object_data[$relation["link"]] = $attachements;
                        break;
                        case "url":
                            if (!$row[$index]) {
                                break;
                            }
                            $attachements = explode("|", $row[$index]);
                            foreach ($attachements as $i => $attachement) {
                                $content = file_get_contents($attachement);
                                if (!$content) {
                                    continue;
                                }
                                $attachements[$i] = UPLOADPATH . $this->base_path . $import_id . "/" . basename($attachement);
                                file_put_contents($attachements[$i], $content);
                                if (!file_exists($attachements[$i])) {
                                    unset($attachements[$i]); 
                                    continue;
                                }
                            }
                            $object_data[$relation["link"]] = $attachements;
                        break;
                        default:
                            $object_data[$relation["link"]] = $row[$index];
                        break;
                    }
                }

                $result = $this->CI->Import_module_model->import_data($object_data, $import_data["id_object"], $import_data["relations"], $user_id, $use_moderation);
                if (!empty($result["errors"])) {
                    log_message('error', 'Validation error', TEMP_DIR . 'logs/import', $import_id . '_' . date('ymd') . 'log');
                    $this->fail_import_data($id, $result["errors"]);
                    ++$import_data["failed"];
                } else {
                    $this->delete_import_data($id);
                    ++$import_data["imported"];

                    if ($import_data['driver']['is_logging']) {
                        $this->save_log($id, array('site_id' => $result['data']['id']));
                    }
                }
                
                foreach ($attachements as $attachement) {
                    unlink($attachement);
                }
                
                if ($import_data["processed"]) {
                    $import_data["processed"]--;
                }
            }

            $save_data = array(
                'failed'    => $import_data["failed"],
                'imported'  => $import_data["imported"],
                'processed' => $import_data["processed"],
            );

            $validate_data = $this->validate_data($import_id, $save_data);
            $this->save_data($import_id, $validate_data["data"]);

            $return["data"]["imported"] = $import_data["imported"];
            $return["data"]["failed"] = $import_data["failed"];
            $return["data"]["processed"] = $import_data["processed"];
        }

        return $return;
    }

    /**
     * Return data for importing as array
     *
     * @param integer $import_id import identifier
     *
     * @return array
     */
    public function get_import_data($import_id)
    {
        $this->DB->select("id, data");
        $this->DB->from(IMPORT_DATA_TABLE);
        $this->DB->where("id_import", $import_id);
        $this->DB->where("errors", "");
        $this->DB->where("status", 1);
        $this->DB->limit($this->import_limit);
        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[$r["id"]] = unserialize($r["data"]);
            }

            return $data;
        }

        return array();
    }

    /**
     * Save import data to data source
     *
     * @param integer $import_data_id import identifier
     * @param array   $import_data    import data
     *
     * @return boolean
     */
    public function save_import_data($import_data_id, $data)
    {
        if (is_null($import_data_id)) {
            $this->DB->insert(IMPORT_DATA_TABLE, $data);
            $import_data_id = $this->DB->insert_id();
        } else {
            $this->DB->where('id', $import_data_id);
            $this->DB->update(IMPORT_DATA_TABLE, $data);
        }

        return $import_data_id;
    }

    /**
     * Approve import data
     *
     * @param integer $import_id import identifier
     *
     * @return boolean
     */
    public function approve_import_data($import_id)
    {
        $data["status"] = 1;
        $this->DB->where("id_import", $import_id);
        $this->DB->update(IMPORT_DATA_TABLE, $data);

        return true;
    }

    /**
     * Decline import data
     *
     * @param integer $import_id import identifier
     *
     * @return boolean
     */
    public function decline_import_data($import_id)
    {
        $this->DB->where("id_import", $import_id);
        $this->DB->where("status", 0);
        $this->DB->delete(IMPORT_DATA_TABLE);

        return true;
    }

    /**
     * Remove import data
     *
     * @param integer $import_id import identifier
     *
     * @return boolean
     */
    public function delete_import_data($import_id)
    {
        $this->DB->where("id", $import_id);
        $this->DB->delete(IMPORT_DATA_TABLE);

        return true;
    }

    /**
     * Fail import data
     *
     * @param integer $import_id import identifier
     * @param array   $errors    errors messages
     *
     * @return boolean
     */
    public function fail_import_data($import_id, $errors)
    {
        $data["errors"] = serialize($errors);
        $this->DB->where("id", $import_id);
        $this->DB->update(IMPORT_DATA_TABLE, $data);

        return true;
    }

    /**
     * Process data of active import by cron
     *
     * @return void
     */
    public function cron_import()
    {
        $subscriptions_list = $this->get_data_list(array("active" => 1));
        foreach ($subscriptions_list as $key => $value) {
            if ($value["scheduler"]["type"] == "manual" ||
               $value["scheduler"]["date_for_cron"] == 0 ||
               $value["scheduler"]["date_for_cron"] > time()) {
                continue;
            }

            $this->make_import_data($value["id"]);
            switch ($value["scheduler"]["type"]) {
                case "intime":
                    $value["scheduler"]["date_for_cron"] = 0;
                break;
                case "cron":
                    $d = $value["scheduler"]["date_for_cron"];
                    switch ($value["scheduler"]["period"]) {
                        case "day":
                            $value["scheduler"]["date_for_cron"] = mktime(date("H", $d), date("i", $d), date("s", $d), date("n", $d), date("j", $d) + 1, date("Y", $d));
                        break;
                        case "week":
                            $value["scheduler"]["date_for_cron"] = mktime(date("H", $d), date("i", $d), date("s", $d), date("n", $d), date("j", $d) + 7, date("Y", $d));
                        break;
                        case "month":
                            $value["scheduler"]["date_for_cron"] = mktime(date("H", $d), date("i", $d), date("s", $d), date("n", $d) + 1, date("j", $d), date("Y", $d));
                        break;
                    }
                break;
            }

            $data = array("scheduler" => $value["scheduler"]);
            $validate_data = $this->validate_data($value["id"], $data);
            if (!empty($validate_data['errors'])) {
                continue;
            }
            $this->save_data($value["id"], $validate_data['data']);
        }
    }

    /**
     * Return objects identifiers from log
     *
     * @param string  $driver_gid driver guid
     * @param integer $object_id  object idnetifier
     * @param integer $system_ids system identifiers
     *
     * @return array
     */
    public function get_object_ids($driver_gid, $object_id, $system_ids)
    {
        $this->DB->select('system_id, site_id');
        $this->DB->from(IMPORT_LOG_TABLE);
        $this->DB->where_in('gid_driver', $driver_gid);
        $this->DB->where_in('id_object', $object_id);
        $this->DB->where_in('system_id', $system_ids);
        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = array();
            foreach ($results as $r) {
                $data['system_id'] = $r['site_id'];
            }

            return $data;
        }

        return array();
    }

    /**
     * Save parse log
     *
     * @param string  $import_data_id import data idnetifier
     * @param integer $object_id      object idnetifier
     * @param integer $system_id      system identifier
     *
     * @return array
     */
    public function save_log($import_data_id, $data)
    {
        if (is_null($import_data_id)) {
            $this->DB->insert(IMPORT_LOG_TABLE, $data);
        } else {
            $this->DB->where('id_import_data', $import_data_id);
            $this->DB->update(IMPORT_LOG_TABLE, $data);
        }
    }

    /**
     * Analyze data
     *
     * @param integer $import_id import identifier
     *
     * @return array
     */
    public function analyze_data($import_id)
    {
        $return = array("errors" => array(), "data" => array(), 'file' => '');

        $import_data = $this->get_data_by_id($import_id, true);

        $model_name = "import_" . $import_data['gid_driver'] . "_model";
        $this->CI->load->model("import/models/drivers/" . $model_name, $model_name);

        if ($this->CI->{$model_name}->need_upload) {
            $upload_data = $this->upload('import_file', $import_id);
            if (!empty($upload_data['errors'])) {
                $return['errors'] = $upload_data['errors'];
            } else {
                $return['file'] = $upload_data['data']['filename'];
            }
        }

        if (empty($return['errors'])) {
            $this->load->model('import/models/Import_driver_model');
            $driver_data = $this->Import_driver_model->get_driver_by_gid($import_data['gid_driver']);

            $analyze_data = $this->CI->{$model_name}->analyze(
                $driver_data['settings'],
                $import_data['settings'],
                $import_data['relations'],
                $return['file'] ? UPLOADPATH . $this->base_path . $import_id . '/' . $return['file'] : '');

            $return = array_merge($return, $analyze_data);
            if (empty($return['errors'])) {
                $analyze_data['data']['total'] += $import_data["processed"];
                $analyze_data['data']["imported"] = 0;
                $analyze_data['data']["failed"] = 0;
                $validate_data = $this->validate_data($import_data, $analyze_data['data']);
                $this->save_data($import_id, $validate_data['data']);
            }
        }

        return $return;
    }
}
