<?php

namespace Pg\Modules\Linker\Models;

/**
 * Linker module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
define('LINKER_TABLE', DB_PREFIX . 'linker');
define('LINKER_TYPES_TABLE', DB_PREFIX . 'linker_types');
define('LINKER_SEPARATED_PREFIX', DB_PREFIX . 'linker_sep_');

/**
 * Linker type Model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Linker
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Linker_type_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Constructor
     *
     * @return Linker_type_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return contact link type object by identifier
     *
     * @param integer $type_id type identifier
     *
     * @return array/false
     */
    public function get_type_by_id($type_id)
    {
        $type_id = intval($type_id);
        if (!$type_id) {
            return false;
        }

        $this->ci->db->select('id, gid, separated, lifetime, unique_type');
        $this->ci->db->from(LINKER_TYPES_TABLE);
        $this->ci->db->where('id', $type_id);

        //_compile_select;
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            $rt = get_object_vars($result[0]);
            $rt["table_name"] = ($rt["separated"]) ? LINKER_SEPARATED_PREFIX . $rt["gid"] : LINKER_TABLE;

            return $rt;
        } else {
            return false;
        }
    }

    /**
     * Return link type object by GUID
     *
     * @param string $type_gid type GUID
     *
     * @return mixed
     */
    public function get_type_by_gid($type_gid)
    {
        $type_gid = preg_replace("/[^a-z_]/", "", strtolower($type_gid));
        if (!$type_gid) {
            return false;
        }

        $this->ci->db->select('id, gid, separated, lifetime, unique_type');
        $this->ci->db->from(LINKER_TYPES_TABLE);
        $this->ci->db->where('gid', $type_gid);

        //_compile_select;
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            $rt = get_object_vars($result[0]);
            $rt["table_name"] = ($rt["separated"]) ? LINKER_SEPARATED_PREFIX . $rt["gid"] : LINKER_TABLE;

            return $rt;
        } else {
            return false;
        }
    }

    /**
     * Return identifier of type object by GUID
     *
     * @param string $type_gid type GUID
     *
     * @return integer/false
     */
    public function get_type_id_by_gid($type_gid)
    {
        $type = $this->get_type_by_gid($type_gid);
        if ($type !== false && !empty($type["id"])) {
            return $type["id"];
        }

        return false;
    }

    /**
     * Create contact link type
     *
     * Link replace type: 'no'/'update'/'noupdate'
     *
     * @param string  $gid         linker type GID [a-z_]
     * @param integer $separated   create separate table for the link
     * @param integer $lifetime    time in sec., 0-unlimited
     * @param string  $unique_type link replace type
     *
     * @return integer
     */
    public function create_type($gid, $separated = 0, $lifetime = 0, $unique_type = 'no')
    {
        $gid = preg_replace("/[^a-z_]/", "", strtolower($gid));

        $id = $this->get_type_id_by_gid($gid);
        if (!$id) {
            $data = array(
                'gid' => $gid,
                'separated' => intval($separated),
            );

            $this->ci->db->insert(LINKER_TYPES_TABLE, $data);
            $id = $this->get_type_id_by_gid($gid);
        }
        if ($separated) {
            $this->create_table($gid);
        }

        return $id;
    }

    /**
     * Remove link type by identifier or GUID
     *
     * @param mixed $id integer ID / string GID
     *
     * @return void
     */
    public function delete_type($id)
    {
        if (!is_int($id)) {
            $id = $this->get_type_id_by_gid($id);
        }
        $this->ci->db->where('id', $id);
        $this->ci->db->delete(LINKER_TYPES_TABLE);

        return;
    }

    /**
     * Create a separated table for link type
     *
     * @param string $gid link type GUID
     *
     * @return string
     */
    public function create_table($gid)
    {
        $table_name = LINKER_SEPARATED_PREFIX . $gid;
        if (!$this->ci->db->table_exists($table_name)) {
            $this->ci->load->dbforge();

            $fields = array(
                'id' => array(
                    'type' => 'INT',
                    'constraint' => 3,
                    'null' => false,
                    'auto_increment' => true,
                ),
                'id_link_1' => array(
                    'type' => 'INT',
                    'constraint' => 3,
                    'null' => false,
                ),
                'id_link_2' => array(
                    'type' => 'INT',
                    'constraint' => 3,
                    'null' => false,
                ),
                'date_add' => array(
                    'type' => 'DATETIME',
                ),
                'sorter' => array(
                    'type' => 'INT',
                    'constraint' => 3,
                    'null' => false,
                ),
            );
            $this->ci->dbforge->add_field($fields);
            $this->ci->dbforge->add_key('id', true);
            $this->ci->dbforge->add_key(array('id_link_1', 'id_link_2'));
            $this->ci->dbforge->add_key('sorter');
            $this->ci->dbforge->create_table($table_name);
        }

        return $table_name;
    }

}
