<?php

namespace Pg\Modules\Listings_booking\Models;

/**
 * Listings module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
define('LISTINGS_BOOKING_PERIODS_TABLE', DB_PREFIX . 'listings_booking');
define('LISTINGS_BOOKING_BLACKLIST_TABLE', DB_PREFIX . 'listings_booking_blacklist');
define('LISTINGS_BOOKING_CACHE_TABLE', DB_PREFIX . 'listings_booking_cache');

/**
 * Listings Booking Model
 *
 * @package     PG_RealEstate
 * @subpackage  Listings
 *
 * @category    models
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings_booking_model extends \Model
{

    const STATUS_OPEN = 'open';
    const STATUS_CLOSE = 'close';
    const STATUS_APPROVE = 'approve';
    const STATUS_WAIT = 'wait';
    const STATUS_DECLINE = 'decline';
    const STATUS_UNCONFIRMED = 'unconfirmed';
    const STATUS_BLACKLISTED = 'blacklisted';
    const PRICE_PERIOD_DAY = 1;
    const PRICE_PERIOD_MONTH = 2;

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Listing object properties in data source
     *
     * @var array
     */
    protected $fields = array(
        'id',
        'id_listing',
        'id_owner',
        'id_user',
        'status',
        'date_start',
        'date_end',
        'guests',
        'price',
        'comment',
        'answer',
        'guest_email',
        'booking_data',
        'date_created',
        'date_modified',
    );

    /**
     * Format settings of booking period
     *
     * @var array
     */
    protected $format_settings = array(
        'use_format' => true,
        'get_user' => true,
        'get_listing' => false,
    );

    /**
     * Available statuses for calendar period
     *
     * @var array
     */
    protected $period_status_arr = array(
        self::STATUS_OPEN,
        self::STATUS_CLOSE,
        self::STATUS_APPROVE,
        self::STATUS_WAIT,
        self::STATUS_BLACKLISTED,
        self::STATUS_DECLINE,
    );

    /**
     * Limit of searching items
     *
     * @var integer
     */
    protected $seacrh_items_limit = 10000;

    /**
     * Booking periods cache
     *
     * @var array
     */
    protected $booking_cache = array();

    /**
     * Class constructor
     *
     * return Listings_booking_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return period object by identifier
     *
     * @param integer $period_id period identifier
     *
     * @return array
     */
    public function get_period_by_id($period_id)
    {
        $this->ci->db->select(implode(', ', $this->fields));
        $this->ci->db->from(LISTINGS_BOOKING_PERIODS_TABLE);
        $this->ci->db->where('id', $period_id);
        $results = $this->ci->db->get()->result_array();
        if (empty($results) || !is_array($results)) {
            return array();
        }
        return current($this->format_periods(array($results[0])));
    }

    /**
     * Save period object to data source
     *
     * @param integer $period_id period identifier
     * @param array   $data      period data
     * @param boolean $booking   turn on of it is user booking
     *
     * @return integer
     */
    public function save_period($period_id, array $data, $booking = false)
    {
        if (!$period_id) {
            $this->ci->load->model('Listings_model');
            $this->ci->Listings_model->set_format_settings('use_format', false);
            $listing = $this->ci->Listings_model->get_listing_by_id($data['id_listing']);
            $this->ci->Listings_model->set_format_settings('use_format', true);
            $data['id_owner'] = $listing['id_user'];
            $data['date_created'] = $data['date_modified'] = date('Y-m-d H:i:s');
            $this->ci->db->insert(LISTINGS_BOOKING_PERIODS_TABLE, $data);
            $period_id = $this->ci->db->insert_id();
        } else {
            $data['date_modified'] = date('Y-m-d H:i:s');
            $this->ci->db->where('id', $period_id);
            $this->ci->db->update(LISTINGS_BOOKING_PERIODS_TABLE, $data);
        }

        if ($booking) {
            $this->ci->load->model('Users_model');

            $this->ci->Users_model->set_format_settings('get_safe', false);

            $this->set_format_settings('get_listing', true);
            $period = $this->get_period_by_id($period_id);
            $this->set_format_settings('get_listing', false);

            $this->ci->Users_model->set_format_settings('get_safe', true);

            // add contacts
            if ($this->ci->pg_module->is_module_installed("users")) {
                $this->ci->load->model('Users_model');
                $this->ci->Users_model->add_contact($period['id_user'], $period['listing']['id_user']);
                $this->ci->Users_model->add_contact($period['listing']['id_user'], $period['id_user']);
            }
            $this->ci->load->model('menu/models/Indicators_model');
            $this->ci->Indicators_model->add('new_booking_item', $period_id, $period['listing']['id_user']);
            $this->ci->Indicators_model->add('all_booking_item', 'period_' . $period_id, $period['listing']['id_user']);

            $mail_data = array(
                'fname' => $period['listing']['user']['fname'],
                'sname' => $period['listing']['user']['sname'],
                'user' => $period['user']['output_name'],
                'name' => $period['listing']['output_name'],
            );

            $this->ci->load->model('Notifications_model');
            $this->ci->Notifications_model->send_notification($period['listing']['user']['email'], 'listing_booking_request', $mail_data, '', $period['listing']['user']['lang_id']);

            $moderation_send_mail = $this->ci->pg_module->get_module_config('listings', 'moderation_send_mail');
            if ($moderation_send_mail) {
                $admin_moderation_emails = $this->ci->pg_module->get_module_config('listings', 'admin_moderation_emails');
                if ($admin_moderation_emails) {
                    unset($mail_data['fname']);
                    unset($mail_data['sname']);
                    $this->ci->Notifications_model->send_notification($admin_moderation_emails, 'admin_booking_request', $mail_data);
                }
            }
        }

        return $period_id;
    }

    /**
     * Save guest period object to data source
     *
     * @param type $data
     * @return type
     */
    public function save_guest_period(array $data = array(), array $user = array())
    {
        if (empty($data) || empty($user)) {
            return false;
        }
        $this->ci->load->model('Listings_model');
        $this->ci->Listings_model->set_format_settings('use_format', false);
        $listing = $this->ci->Listings_model->get_listing_by_id($data['id_listing']);
        $this->ci->Listings_model->set_format_settings('use_format', true);

        if (!empty($data['comment'])) {
            $data['comment'] = trim(strip_tags($data['comment']));
        }

        $data['id_owner'] = $listing['id_user'];
        $data['date_created'] = $data['date_modified'] = date('Y-m-d H:i:s');
        $data['status'] = self::STATUS_UNCONFIRMED;
        $data['guest_email'] = $user['email'];
        $booking_data = array(
            'confirmation_code' => md5($user['email'] . $data['date_created']),
            'phone_number' => $user['phone'],
            'fname' => $user['fname'],
            'sname' => $user['sname'],
            'lang_id' => $this->ci->pg_language->current_lang_id,
        );
        $data['booking_data'] = serialize($booking_data);

        $this->ci->db->insert(LISTINGS_BOOKING_PERIODS_TABLE, $data);
        $period_id = $this->ci->db->insert_id();

        $mail_data = array(
            'fname' => $user['fname'],
            'sname' => $user['sname'],
            'confirmation_link' => site_url() . 'listings_booking/guest_confirm_booking/' . $period_id . '/' . $booking_data['confirmation_code'],
        );

        if ($period_id) {
            $this->ci->load->model('Notifications_model');
            $this->ci->Notifications_model->send_notification(
                $user['email'], 'guest_booking_request', $mail_data, '', $this->ci->pg_language->current_lang_id
            );
            return true;
        } else {
            return false;
        }
    }

    /**
     * Guest period confirmation
     *
     * @param type $period_id
     */
    public function confirm_guest_period($period_id = null, $confirmation_code = '')
    {
        $this->set_format_settings('get_listing', true);
        $period = $this->get_period_by_id($period_id);
        $this->set_format_settings('get_listing', false);

        if (empty($period['booking_data'])) {
            return false;
        }

        if ($period['booking_data']['confirmation_code'] == $confirmation_code) {
            $period['booking_data']['old_confirmation_code'] = $period['booking_data']['confirmation_code'];
            $data['booking_data'] = $period['booking_data'];
            $data['booking_data']['confirmation_code'] = md5($confirmation_code . rand(0, 9999) . time());
            $data['booking_data'] = serialize($data['booking_data']);

            $data['status'] = self::STATUS_WAIT;
            $data['id_user'] = 0;
            $this->save_period($period_id, $data);

            $this->ci->load->model('menu/models/Indicators_model');
            $this->ci->Indicators_model->add('new_booking_item', $period_id, $period['listing']['id_user']);
            $this->ci->Indicators_model->add('all_booking_item', 'period_' . $period_id, $period['listing']['id_user']);

            $guest_output_string = $period['booking_data']['fname'] . ' '
                . $period['booking_data']['sname'] . ' <' . $period['guest_email'] . '>';

            $mail_data = array(
                'fname' => $period['listing']['user']['fname'],
                'sname' => $period['listing']['user']['sname'],
                'user' => $guest_output_string,
                'name' => $period['listing']['output_name'],
            );
            $listing_owner_email = isset($period['listing']['user']['email']) ? $period['listing']['user']['email'] : $period['listing']['user']['contact_email'];
            $this->ci->load->model('Notifications_model');
            $this->ci->Notifications_model->send_notification($listing_owner_email, 'listing_booking_request', $mail_data, '', $period['listing']['user']['lang_id']);

            $moderation_send_mail = $this->ci->pg_module->get_module_config('listings', 'moderation_send_mail');
            if ($moderation_send_mail) {
                $admin_moderation_emails = $this->ci->pg_module->get_module_config('listings', 'admin_moderation_emails');
                if ($admin_moderation_emails) {
                    unset($mail_data['fname']);
                    unset($mail_data['sname']);
                    $this->ci->Notifications_model->send_notification($admin_moderation_emails, 'admin_booking_request', $mail_data);
                }
            }
            return 'confirm';
        } elseif (isset($period['booking_data']['old_confirmation_code']) && $period['booking_data']['old_confirmation_code'] == $confirmation_code) {
            return 'already_confirmed';
        } else {
            return false;
        }
    }

    /**
     * Remove period object from data source
     *
     * @param integer $period_id period identifier
     *
     * @return void
     */
    public function delete_period($period_id)
    {
        $this->ci->db->where('id', $period_id);
        $this->ci->db->delete(LISTINGS_BOOKING_PERIODS_TABLE);

        $this->ci->load->model('menu/models/Indicators_model');
        $this->ci->Indicators_model->delete('new_booking_item', $period_id, true);
        $this->ci->Indicators_model->delete('all_booking_item', 'period_' . $period_id, true);
    }

    /**
     * Approve period object
     *
     * @param integer $period_id period identifier
     *
     * @return void
     */
    public function approve_period($period_id)
    {
        $this->setPeriodStatus($period_id, self::STATUS_APPROVE);

        $this->set_format_settings('get_listing', true);
        $period = $this->get_period_by_id($period_id);
        $this->set_format_settings('get_listing', false);

        $this->ci->load->model('menu/models/Indicators_model');
        $this->ci->Indicators_model->delete('new_booking_item', $period_id, true);
        $this->ci->Indicators_model->delete('all_booking_item', 'period_' . $period_id, true);
        $this->ci->Indicators_model->add('booking_status_item', $period['id_user'], $period['id_user']);
        $this->ci->Indicators_model->add('all_booking_item', 'user_' . $period['id_user'], $period['id_user']);

        $mail_data = array(
            'fname' => $period['user']['fname'],
            'sname' => $period['user']['sname'],
            'user' => $period['listing']['user']['output_name'],
            'name' => $period['listing']['output_name'],
        );

        $this->load->model('Notifications_model');
        $this->ci->Notifications_model->send_notification($period['user']['email'], 'listing_booking_approve', $mail_data, '', $period['user']['lang_id']);

        $moderation_send_mail = $this->ci->pg_module->get_module_config('listings', 'moderation_send_mail');
        if ($moderation_send_mail) {
            $admin_moderation_emails = $this->ci->pg_module->get_module_config('listings', 'admin_moderation_emails');
            if ($admin_moderation_emails) {
                unset($mail_data['fname']);
                unset($mail_data['sname']);
                $this->ci->Notifications_model->send_notification($admin_moderation_emails, 'admin_booking_approve', $mail_data);
            }
        }
    }

    /**
     * Decline period object
     *
     * @param integer $period_id period identifier
     *
     * @return void
     */
    public function decline_period($period_id)
    {
        $this->setPeriodStatus($period_id, self::STATUS_DECLINE);

        $this->set_format_settings('get_listing', true);
        $period = $this->get_period_by_id($period_id);
        $this->set_format_settings('get_listing', false);

        $this->ci->load->model('menu/models/Indicators_model');
        $this->ci->Indicators_model->delete('new_booking_item', $period_id, true);
        $this->ci->Indicators_model->delete('all_booking_item', 'period_' . $period_id, true);
        $this->ci->Indicators_model->add('booking_status_item', $period['id_user'], $period['id_user']);
        $this->ci->Indicators_model->add('all_booking_item', 'user_' . $period['id_user'], $period['id_user']);

        $mail_data = array(
            'fname' => $period['user']['fname'],
            'sname' => $period['user']['sname'],
            'user' => $period['listing']['user']['output_name'],
            'name' => $period['listing']['output_name'],
        );

        $this->ci->load->model('Notifications_model');
        $this->ci->Notifications_model->send_notification($period['user']['email'], 'listing_booking_decline', $mail_data, '', $period['user']['lang_id']);

        $moderation_send_mail = $this->ci->pg_module->get_module_config('listings', 'moderation_send_mail');
        if ($moderation_send_mail) {
            $admin_moderation_emails = $this->ci->pg_module->get_module_config('listings', 'admin_moderation_emails');
            if ($admin_moderation_emails) {
                unset($mail_data['fname']);
                unset($mail_data['sname']);
                $this->ci->Notifications_model->send_notification($admin_moderation_emails, 'admin_booking_decline', $mail_data);
            }
        }
    }

    /**
     * Approve guest period object
     *
     * @param integer $period_id period identifier
     *
     * @return void
     */
    public function approve_guest_period($period_id = null, $data = array())
    {
        $this->set_format_settings('get_listing', true);
        $period = $this->get_period_by_id($period_id);
        $this->set_format_settings('get_listing', false);

        $this->ci->load->model('menu/models/Indicators_model');
        $this->ci->Indicators_model->delete('new_booking_item', $period_id, true);
        $this->ci->Indicators_model->delete('all_booking_item', 'period_' . $period_id, true);

        $this->ci->view->assign('period', $period);
        $this->ci->view->assign('view_mode', 'letter');
        $mail_data['content'] = $this->ci->view->fetch('guest_order_view');
        $this->load->model('Notifications_model');
        $this->ci->Notifications_model->send_notification($period['guest_email'], 'guest_booking_approve', $mail_data, '', 1); // потом вставить это $period['booking_data']['lang_id']);

        $moderation_send_mail = $this->ci->pg_module->get_module_config('listings', 'moderation_send_mail');
        if ($moderation_send_mail) {
            $admin_moderation_emails = $this->ci->pg_module->get_module_config('listings', 'admin_moderation_emails');
            if ($admin_moderation_emails) {
                unset($mail_data['fname']);
                unset($mail_data['sname']);
                $this->ci->Notifications_model->send_notification($admin_moderation_emails, 'admin_booking_approve', $mail_data);
            }
        }
    }

    /**
     * Approve guest period object
     *
     * @param integer $period_id period identifier
     *
     * @return void
     */
    public function decline_guest_period($period_id = null, $data = array())
    {
        $this->set_format_settings('get_listing', true);
        $period = $this->get_period_by_id($period_id);
        $this->set_format_settings('get_listing', false);

        $this->ci->load->model('menu/models/Indicators_model');
        $this->ci->Indicators_model->delete('new_booking_item', $period_id, true);
        $this->ci->Indicators_model->delete('all_booking_item', 'period_' . $period_id, true);

        $this->ci->view->assign('period', $period);
        $this->ci->view->assign('view_mode', 'letter');
        $mail_data = array(
            'fname' => $period['booking_data']['fname'],
            'sname' => $period['booking_data']['sname'],
            'decline_reason' => $period['answer'],
            'user' => $period['user']['output_name'],
            'name' => $period['listing']['output_name'],
        );
        $this->load->model('Notifications_model');
        $this->ci->Notifications_model->send_notification($period['guest_email'], 'guest_booking_decline', $mail_data, 'guest_booking_decline', $period['booking_data']['lang_id']); // потом вставить это $period['booking_data']['lang_id']);

        $moderation_send_mail = $this->ci->pg_module->get_module_config('listings', 'moderation_send_mail');
        if ($moderation_send_mail) {
            $admin_moderation_emails = $this->ci->pg_module->get_module_config('listings', 'admin_moderation_emails');
            if ($admin_moderation_emails) {
                unset($mail_data['fname']);
                unset($mail_data['sname']);
                $this->ci->Notifications_model->send_notification($admin_moderation_emails, 'admin_booking_decline', $mail_data);
            }
        }
    }

    /**
     * Return search criteria by filters data
     *
     * @param array $filters filters data
     *
     * @return array
     */
    protected function getSearchCriteria($filters)
    {
        $params = array();

        // By keyword
        if (!empty($filters['keyword'])) {
            $boolean_mode = false;

            if (isset($filters['keyword_mode'])) {
                $boolean_mode = true;
                unset($filters['keyword_mode']);
            }

            $filters['keyword'] = trim($filters['keyword']);

            $fe_criteria = array();

            $this->ci->load->model('Listings_model');

            $this->ci->load->model('Field_editor_model');
            $this->ci->load->model('listings/models/Listings_field_editor_model');
            $property_type_gids = $this->ci->Listings_field_editor_model->get_field_editor_types();
            foreach ($property_type_gids as $property_type_gid) {
                $this->ci->Field_editor_model->initialize($property_type_gid);
                $temp_criteria = $this->ci->Field_editor_model->return_fulltext_criteria($filters['keyword'], $boolean_mode);
                $field_editor_table = constant('LISTINGS_' . strtoupper($property_type_gid) . '_TABLE');
                $fe_table_name = substr($field_editor_table, strlen(DB_PREFIX));
                if (!empty($temp_criteria[$fe_table_name]['where_sql'])) {
                    $fe_criteria[] = str_replace('search_field', $field_editor_table . '.search_field', $temp_criteria[$fe_table_name]['where_sql']);
                }
            }
            if (!empty($fe_criteria)) {
                $fe_criteria = array('where_sql' => array('(' . implode(' OR ', array_unique($fe_criteria)) . ')'));
                if (isset($filters['field_editor'])) {
                    $filters['field_editor'][0] = array_merge($filters['field_editor'][0], $property_type_gids);
                    $filters['field_editor'][1] = array_merge($filters['field_editor'][1], $fe_criteria);
                } else {
                    $filters['field_editor'] = array($property_type_gids, $fe_criteria);
                }
            }
        }

        $fields = array_flip($this->fields);
        foreach ((array) $filters as $filter_name => $filter_data) {
            if (empty($filter_data)) {
                continue;
            }
            switch ($filter_name) {
                // By guest_emails
                case 'where':
                    if (is_array($filter_data)) {
                        foreach ($filter_data as $field_name => $field_value) {
                            $params = array_merge_recursive($params, array('where' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.' . $field_name => $field_value)));
                        }
                    }
                    break;
                // By ids
                case 'ids':
                    if (is_array($filter_data)) {
                        $params = array_merge_recursive($params, array('where_in' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.id' => $filter_data)));
                    } else {
                        $params = array_merge_recursive($params, array('where' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.id' => $filter_data)));
                    }
                    break;
                // By ids
                case 'not_ids':
                    if (is_array($filter_data)) {
                        $params = array_merge_recursive($params, array('where_not_in' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.id' => $filter_data)));
                    } else {
                        $params = array_merge_recursive($params, array('where' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.id' => '!= ' . $filter_data)));
                    }
                    break;
                // By listings
                case 'listings':
                    if (is_array($filter_data)) {
                        $params = array_merge_recursive($params, array('where_in' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.id_listing' => $filter_data)));
                    } else {
                        $params = array_merge_recursive($params, array('where' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.id_listing' => $filter_data)));
                    }
                    break;
                // By users
                case 'users':
                    if (is_array($filter_data)) {
                        $params = array_merge_recursive($params, array('where_in' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.id_user' => $filter_data)));
                    } else {
                        $params = array_merge_recursive($params, array('where' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.id_user' => $filter_data)));
                    }
                    break;
                // By users
                case 'not_user':
                    $params = array_merge_recursive($params, array('where' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.id_user !=' => $filter_data)));
                    break;
                // By owner
                case 'owners':
                    if (is_array($filter_data)) {
                        $params = array_merge_recursive($params, array('where_in' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.id_owner' => $filter_data)));
                    } else {
                        $params = array_merge_recursive($params, array('where' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.id_owner' => $filter_data)));
                    }
                    break;
                // By users
                case 'not_owner':
                    $params = array_merge_recursive($params, array('where' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.id_owner !=' => $filter_data)));
                    break;
                // By users
                case 'status':
                    if (is_array($filter_data)) {
                        $params = array_merge_recursive($params, array('where_in' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.status' => $filter_data)));
                    } else {
                        $params = array_merge_recursive($params, array('where' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.status' => $filter_data)));
                    }
                    break;
                // By field editor
                case 'field_editor':
                    if (empty($filter_data)) {
                        continue;
                    }
                    foreach ($filter_data[0] as $key => $property_type_gid) {
                        $filter_data[0][$key] = constant('LISTINGS_' . strtoupper($property_type_gid) . '_TABLE');
                    }
                    $filter_data[0] = array_unique($filter_data[0]);
                    foreach ($filter_data[0] as $key => $field_editor_table) {
                        $filter_data[0][$key] = array('table' => $field_editor_table, 'condition' => LISTINGS_BOOKING_PERIODS_TABLE . '.id_listing=' . $field_editor_table . '.id_listing', 'type' => 'LEFT');
                    }
                    $params = array_merge_recursive($params, array('join' => $filter_data[0]), $filter_data[1]);
                    break;
                // By field
                default:
                    if (strpos($filter_name, ' ') !== false) {
                        $field_name = current(explode(' ', $filter_name));
                    } else {
                        $field_name = $filter_name;
                    }
                    if (!isset($fields[$field_name])) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array(LISTINGS_BOOKING_PERIODS_TABLE . '.' . $filter_name => $filter_data)));
                    break;
            }
        }
        return $params;
    }

    /**
     * Return periods objects in data source as array
     *
     * @param integer $page     page of results
     * @param string  $limits   results limit
     * @param array   $order_by sorting data
     * @param array   $params   sql criteria
     *
     * @return array
     */
    protected function getPeriodsList($page = null, $limits = null, $order_by = null, array $params = array())
    {
        $this->ci->db->select(LISTINGS_BOOKING_PERIODS_TABLE . '.' . implode(', ' . LISTINGS_BOOKING_PERIODS_TABLE . '.', $this->fields));
        $this->ci->db->from(LISTINGS_BOOKING_PERIODS_TABLE);

        if (isset($params['join']) && is_array($params['join']) && count($params['join'])) {
            foreach ($params['join'] as $join) {
                $this->ci->db->join($join['table'], $join['condition'], $join['type']);
            }
        }

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params['where_in']) && is_array($params['where_in']) && count($params['where_in'])) {
            foreach ($params['where_in'] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params['where_not_in']) && is_array($params['where_not_in']) && count($params['where_not_in'])) {
            foreach ($params['where_not_in'] as $field => $value) {
                $this->ci->db->where_not_in($field, $value);
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields)) {
                    $this->ci->db->order_by($field . ' ' . $dir);
                }
            }
        } elseif ($order_by) {
            $this->ci->db->order_by($order_by);
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($limits, $limits * ($page - 1));
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $this->format_periods($results);
        } else {
            return array();
        }
    }

    /**
     * Return number of periods objects in data source
     *
     * @param array $params sql criteria
     *
     * @return integer
     */
    protected function fetchPeriodsCount($params = null)
    {
        $this->ci->db->select('COUNT(*) AS cnt');
        $this->ci->db->from(LISTINGS_BOOKING_PERIODS_TABLE);

        if (isset($params['join']) && is_array($params['join']) && count($params['join'])) {
            foreach ($params['join'] as $join) {
                $this->ci->db->join($join['table'], $join['condition'], $join['type']);
            }
        }

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params['where_in']) && is_array($params['where_in']) && count($params['where_in'])) {
            foreach ($params['where_in'] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->ci->db->where($value);
            }
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {

            return intval($results[0]['cnt']);
        }

        return 0;
    }

    /**
     * Return filtered calendar periods as array
     *
     * @param array   $filters       filter data
     * @param integer $page          page of results
     * @param integer $items_on_page items per page
     * @param string  $order_by      sorting data
     *
     * @return array
     */
    public function get_periods_list($filters = array(), $page = null, $items_on_page = null, $order_by = null)
    {
        $params = $this->getSearchCriteria($filters);
        return $this->getPeriodsList($page, $items_on_page, $order_by, $params);
    }

    public function getPeriodById($id)
    {
        $list = $this->get_periods_list(['where' => ['id' => $id]]);
        if (!empty($list)) {
            return $list[0];
        } else {
            return [];
        }
    }

    /**
     * Return number of filtered calendar periods
     *
     * @param array $filters filter data
     * @return array
     */
    public function getPeriodsCount(array $filters = array())
    {
        $params = $this->getSearchCriteria($filters);
        return $this->fetchPeriodsCount($params);
    }

    /**
     * Return number of filtered calendar periods grouped by statuses
     * @param array $statuses
     * @param int $owner
     * @return array
     */
    public function getPeriodsCounts(array $statuses, $owner = null)
    {
        $result = [];
        if ($owner) {
            $filter = ['not_user' => $owner, 'owners' => $owner,];
        } else {
            $filter = [];
        }
        foreach ($statuses as $status) {
            $filter['status'] = $status;
            $result[$status] = $this->getPeriodsCount($filter);
        }
        return $result;
    }

    /**
     * Return number of filtered calendar periods grouped by statuses
     * @param array $statuses
     * @param int $owner
     * @return array
     */
    public function getRequestsCounts(array $statuses, $user_id = null)
    {
        $result = [];
        if ($user_id) {
            $filter = ['users' => $user_id, 'not_owner' => $user_id,];
        } else {
            $filter = [];
        }
        foreach ($statuses as $status) {
            $filter['status'] = $status;
            $result[$status] = $this->getPeriodsCount($filter);
        }
        return $result;
    }

    /**
     * Return filtered calendar periods by key
     *
     * @param array   $filters       filter data
     * @param integer $page          page of results
     * @param integer $items_on_page items per page
     * @param string  $order_by      sorting data
     *
     * @return array
     */
    public function get_periods_list_by_key($filters = array(), $page = null, $items_on_page = null, $order_by = null)
    {
        $params = $this->getSearchCriteria($filters);
        $periods_data = $this->getPeriodsList($page, $items_on_page, $order_by, $params);
        $return_data = [];
        foreach ($periods_data as $data) {
            if (!isset($return_data[$data['id_listing']])) {
                $return_data[$data['id_listing']] = array();
            }
            $return_data[$data['id_listing']][] = $data;
        }

        return $return_data;
    }

    /**
     * Change format settings for calendar periods objects
     *
     * @param string $name  parameter name
     * @param mixed  $value parameter value
     *
     * @return void
     */
    public function set_format_settings($name, $value = false)
    {
        if (!is_array($name)) {
            $name = array($name => $value);
        }
        if (empty($name)) {
            return;
        }
        foreach ($name as $key => $item) {
            $this->format_settings[$key] = $item;
        }
    }

    /**
     * Validate calendar period object
     *
     * @param integer $period_id  period identifier
     * @param array   $data       period data
     * @param boolean $is_booking turn on if it is booking request
     *
     * @return array
     */
    public function validate_period($period_id, $data, $is_booking = false, $is_guest_booking = false, $is_special_price = false)
    {
        $return = array('errors' => array(), 'data' => array(), 'intersect_periods' => array());

        if ($period_id) {
            $this->set_format_settings('use_format', false);
            $period = $this->get_period_by_id($period_id);
            $this->set_format_settings('use_format', true);
        } else {
            $period = array();
        }

        if (isset($data['id'])) {
            $return['data']['id'] = intval($data['id']);
            if (empty($return['data']['id'])) {
                unset($return['data']['id']);
            }
        }

        if (isset($data['id_listing'])) {
            $return['data']['id_listing'] = intval($data['id_listing']);
            if (empty($return['data']['id_listing'])) {
                $return['errors'][] = l('error_empty_calendar_listing', 'listings');
            } else {
                $period['id_listing'] = $return['data']['id_listing'];
            }
        } elseif (!$period_id) {
            $return['errors'][] = l('error_empty_calendar_listing', 'listings');
        }

        if ($is_guest_booking === true) {
            $return['data']['id_user'] = 0;
        } else {
            if (isset($data['id_user'])) {
                $return['data']['id_user'] = intval($data['id_user']);
                if (empty($return['data']['id_user'])) {
                    $return['errors'][] = l('error_empty_calendar_user', 'listings');
                } else {
                    $period['id_user'] = $return['data']['id_user'];
                }
            } elseif (!$period_id) {
                $return['errors'][] = l('error_empty_calendar_user', 'listings');
            }
        }

        if (isset($data['date_start'])) {
            $value = strtotime($data['date_start']);
            if ($value > 0) {
                $period['date_start'] = $return['data']['date_start'] = date('Y-m-d', $value);
            } else {
                $return['errors'][] = l('error_empty_calendar_date_start', 'listings');
            }
        } else {
            $return['errors'][] = l('error_empty_calendar_date_start', 'listings');
        }

        if (isset($data['date_end'])) {
            $value = strtotime($data['date_end']);
            if ($value > 0) {
                $period['date_end'] = $return['data']['date_end'] = date('Y-m-d', $value);
            } else {
                $return['errors'][] = l('error_empty_calendar_date_end', 'listings');
            }
        } else {
            $return['errors'][] = l('error_empty_calendar_date_end', 'listings');
        }

        if (isset($data['status'])) {
            $return['data']['status'] = $data['status'];
            if (empty($return['data']['status'])) {
                $return['errors'][] = l('error_empty_calendar_status', 'listings');
            } elseif (!in_array($data['status'], $this->period_status_arr)) {
                $return['errors'][] = l('error_invalid_calendar_status', 'listings');
            }
        } elseif (!$period_id) {
            if ($is_booking) {
                $return['data']['status'] = self::STATUS_WAIT;
            } else {
                $return['errors'][] = l('error_empty_calendar_status', 'listings');
            }
        }

        if (isset($period['date_start']) && isset($period['date_end'])) {
            if (strtotime($period['date_start']) > strtotime($period['date_end'])) {
                $return['errors'][] = l('error_invalid_calendar_period', 'listings');
            } elseif (isset($period['id_listing']) && !empty($period['id_listing'])) {
                $this->ci->load->model('Listings_model');
                $this->ci->Listings_model->set_format_settings('use_format', false);
                $listing = $this->ci->Listings_model->get_listing_by_id($period['id_listing']);
                $this->ci->Listings_model->set_format_settings('use_format', true);

                switch ($listing['price_period']) {
                    case self::PRICE_PERIOD_DAY:
                        $n = (strtotime($period['date_end']) + 86400 - strtotime($period['date_start'])) / 86400;
                        break;
                    case self::PRICE_PERIOD_MONTH:
                        if (class_exists('DateTime')) {
                            $datetime = new \DateTime($period['date_end']);
                            $n = $datetime->diff(new \DateTime($period['date_start']))->format('%m');
                        } else {
                            $n = ((strtotime($period['date_end']) - strtotime($period['date_start'])) / (86400 * 30));
                        }
                        break;
                }
                if (!$is_special_price) {
                    if ($listing['calendar_period_min'] > 0 && $n < $listing['calendar_period_min']) {
                        $return['errors'][] = str_replace(array('[value]', '[unit]'), array($listing['calendar_period_min'], ld_option('price_period_unit', 'listings', $listing['price_period'])), l('error_calendar_period_min', 'listings'));
                    }
                    if ($listing['calendar_period_max'] > 0 && $n > $listing['calendar_period_max']) {
                        $return['errors'][] = str_replace(array('[value]', '[unit]'), array($listing['calendar_period_max'], ld_option('price_period_unit', 'listings', $listing['price_period'])), l('error_calendar_period_max', 'listings'));
                    }
                }
                if (isset($return['data']['status']) && $return['data']['status'] == self::STATUS_OPEN) {
                    $this->set_format_settings('get_user', false);
                    $intersect_periods = $this->is_intersect_exists_periods($period['id_listing'], $period['id_user'], $period['date_start'], $period['date_end'], $period_id);
                    if (!empty($intersect_periods)) {
                        if ($listing['price_period'] == self::PRICE_PERIOD_MONTH) {
                            $return['errors'][] = l('error_intersect_calendar_periods', 'listings');
                        } else {
                            $return['intersect_periods'] = $intersect_periods;
                        }
                    }
                } else {
                    $is_intersect = $this->is_intersect_exists_orders($period['id_listing'], 0, $period['date_start'], $period['date_end'], $period_id);
                    if ($is_intersect) {
                        $return['errors'][] = l('error_intersect_calendar_periods', 'listings');
                    }
                }
            }
        }

        if (isset($data['guests'])) {
            $return['data']['guests'] = intval($data['guests']);
        }

        if (isset($data['price'])) {
            $return['data']['price'] = floatval($data['price']);
        }

        if (isset($data['comment'])) {
            $return['data']['comment'] = trim(strip_tags($data['comment']));
        }

        if (isset($data['answer'])) {
            $return['data']['answer'] = trim(strip_tags($data['answer']));
        }

        if (isset($data['date_created'])) {
            $value = strtotime($data['date_created']);
            if ($value > 0) {
                $return['data']['date_created'] = date('Y-m-d', $value);
            }
        }

        if (isset($data['date_modified'])) {
            $value = strtotime($data['date_modified']);
            if ($value > 0) {
                $return['data']['date_modified'] = date('Y-m-d', $value);
            }
        }

        return $return;
    }

    /**
     * Format data of calendar period object
     *
     * @param array $data data set of calendar period
     *
     * @return array
     */
    public function format_period($data)
    {
        return current($this->format_periods(array($data)));
    }

    /**
     * Format data of calendar periods objects
     *
     * @param array $data data set of calendar periods
     *
     * @return array
     */
    public function format_periods($data)
    {
        if (!$this->format_settings['use_format']) {
            return $data;
        }

        $users_search = array();
        $listings_search = array();

        foreach ($data as $key => $period) {
            $period['price'] *= 1;

            $period['status_str'] = ld_option('booking_status', 'listings', $period['status']);
            if (isset($period['guests']) && $period['guests']) {
                $period['guests_str'] = ld_option('booking_guests', 'listings', $period['guests']);
            }

            if (!empty($period['booking_data'])) {
                $period['booking_data'] = unserialize($period['booking_data']);
            }

            if (!empty($period['date_start'])) {
                $period['date_start'] = strftime("%Y-%m-%d", strtotime($period['date_start']));
                $period['date_start_tstamp'] = strtotime($period['date_start']);
            }

            if (!empty($period['date_end'])) {
                $period['date_end'] = strftime("%Y-%m-%d", strtotime($period['date_end']));
                $period['date_end_tstamp'] = strtotime($period['date_end']);
            }

            if ($period['id_user'] != $period['id_owner']) {
                switch ($period['status']) {
                    case self::STATUS_APPROVE:
                        $period['status_str'] = l('booking_status_approved', 'listings');
                        break;
                    case self::STATUS_DECLINE:
                        $period['status_str'] = l('booking_status_declined', 'listings');
                        break;
                    case self::STATUS_WAIT:
                        $period['status_str'] = l('booking_status_wait', 'listings');
                        break;
                }
            }

            //  get listing
            if ($this->format_settings['get_listing']) {
                $listings_search[] = $period['id_listing'];
            }

            //  get user
            if ($this->format_settings['get_user']) {
                $users_search[] = $period['id_user'];
            }

            $data[$key] = $period;
        }

        // get user
        if ($this->format_settings['get_user'] && !empty($users_search) && $this->ci->pg_module->is_module_installed("users")) {
            $this->ci->load->model('Users_model');
            $users_data = $this->ci->Users_model->get_users_list_by_key(null, null, null, array(), $users_search);
            foreach ($data as $key => $period) {
                $data[$key]['user'] = (isset($users_data[$period['id_user']])) ?
                    $users_data[$period['id_user']] : $this->ci->Users_model->format_default_user($period['id_user']);
            }
        }

        // get listing
        if ($this->format_settings['get_listing'] && !empty($listings_search)) {
            $this->ci->load->model('Listings_model');
            $listings_data = $this->ci->Listings_model->get_listings_list_by_key(array('ids' => $listings_search));
            foreach ($data as $key => $period) {
                $data[$key]['listing'] = (isset($listings_data[$period['id_listing']])) ?
                    $listings_data[$period['id_listing']] : $this->ci->Listings_model->format_default_listing($period['id_listing']);
            }
        }

        return $data;
    }

    /**
     * Check intersections between booking order and existed orders
     *
     * @param integer $listing_id listing identifier
     * @param integer $user_id    listing identifier
     * @param string  $date_start start date of period
     * @param string  $date_end   end date of period
     * @param integer $period_id  period identifier
     *
     * @return boolean
     */
    public function is_intersect_exists_orders($listing_id, $user_id, $date_start, $date_end, $period_id = null)
    {
        $params = array();
        $params['where']['id_listing'] = $listing_id;
        $params['where']['date_start <='] = $date_end;
        $params['where']['date_end >'] = $date_start;
        $params['where']['status'] = self::STATUS_APPROVE;
        if ($user_id) {
            $params['where']['id_user'] = $user_id;
        }
        if ($period_id) {
            $params['where']['id !='] = $period_id;
        }

        return $this->getPeriodsCount($params) > 0;
    }

    /**
     * Check intersections between calendar period and existed periods
     *
     * @param integer $listing_id listing identifier
     * @param integer $user_id    listing identifier
     * @param string  $date_start start date of period
     * @param string  $date_end   end date of period
     * @param integer $period_id  period identifier
     *
     * @return boolean
     */
    public function is_intersect_exists_periods($listing_id, $user_id, $date_start, $date_end, $period_id = null)
    {
        $params = array();
        $params['where']['id_listing'] = $listing_id;
        $params['where']['date_start <='] = $date_end;
        $params['where']['date_end >='] = $date_start;
        $params['where']['status'] = self::STATUS_OPEN;
        if ($user_id) {
            $params['where']['id_user'] = $user_id;
        }
        if ($period_id) {
            $params['where']['id !='] = $period_id;
        }
        return $this->get_periods_list($params);
    }

    /**
     * Return identifiers of booking listings
     *
     * @param array $data criteria data
     *
     * @return array
     */
    public function get_booking_listings($data)
    {
        if (empty($data['date_start']) || empty($data['date_end'])) {
            return array();
        }

        $hash = array();

        if (isset($data['date_start']) && !empty($data['date_start'])) {
            $hash[] = 'start:' . $data['date_start'];
        }

        if (isset($data['date_end']) && !empty($data['date_end'])) {
            $hash[] = 'end:' . $data['date_end'];
        }

        if (isset($data['guests']) && !empty($data['guests'])) {
            $hash[] = 'guests:' . $data['guests'];
        }

        $hash = md5(implode(' ', $hash));

        if (!isset($this->booking_cache[$hash])) {
            $cache_period = $this->ci->pg_module->get_module_config('listings', 'booking_cache_period');
            if ($cache_period) {
                $this->ci->db->select('data');
                $this->ci->db->from(LISTINGS_BOOKING_CACHE_TABLE);
                $this->ci->db->where('hash', $hash);
                $this->ci->db->where('expire >', time());
                $results = $this->ci->db->get()->result_array();
                if (!empty($results) && is_array($results)) {
                    $this->booking_cache[$hash] = $results[0]['data'] ? (array) unserialize($results[0]['data']) : array();

                    return $this->booking_cache[$hash];
                }
            }

            $this->ci->db->select('id_listing');
            $this->ci->db->from(LISTINGS_BOOKING_PERIODS_TABLE);
            $this->ci->db->where_in('status', array(self::STATUS_CLOSE, self::STATUS_APPROVE/* , self::STATUS_WAIT */));

            if (isset($data['date_start']) && !empty($data['date_start'])) {
                $this->ci->db->where('date_end >', $data['date_start']);
            }

            if (isset($data['date_end']) && !empty($data['date_end'])) {
                $this->ci->db->where('date_start <=', $data['date_end']);
            }

            if (isset($data['guests']) && !empty($data['guests'])) {
                $this->ci->db->or_where('id_user=id_owner', null, false);
                $this->ci->db->where('status', self::STATUS_OPEN);
                $this->ci->db->where('guests <', $data['guests']);
            }

            $this->ci->db->limit($this->seacrh_items_limit);

            $results = $this->ci->db->get()->result_array();
            if (!empty($results) && is_array($results)) {
                foreach ($results as $i => $row) {
                    $results[$i] = $row['id_listing'];
                }
                $data = array_unique($results);
            } else {
                $data = array();
            }

            if ($cache_period) {
                $save_data = array('data' => serialize($data), 'expire' => time() + $cache_period);
                $this->ci->db->where('hash', $hash);
                $this->ci->db->update(LISTINGS_BOOKING_CACHE_TABLE, $save_data);
                if (!$this->ci->db->affected_rows()) {
                    $save_data['hash'] = $hash;
                    $this->ci->db->insert(LISTINGS_BOOKING_CACHE_TABLE, $save_data);
                }
            }

            $this->booking_cache[$hash] = $data;
        }

        return $this->booking_cache[$hash];
    }

    /**
     * Validate guest data for saving to data source
     *
     * @param array   $data      user data
     *
     * @return array
     */
    public function validateGuest($data = array(), $id_listing_owner = null)
    {
        $return = array("errors" => array(), "data" => array());
        $this->ci->config->load("reg_exps", true);

        // email
        if (isset($data['email'])) {
            $return["data"]["email"] = trim(strip_tags($data["email"]));

            $email_expr = $this->ci->config->item("email", "reg_exps");
            if (empty($return["data"]["email"]) || !preg_match($email_expr, $return["data"]["email"])) {
                $return["errors"][] = l("error_email_incorrect", "users");
            }

            $is_blacklisted = $this->is_email_blacklisted_by_user($return["data"]["email"], $id_listing_owner);

            if ($is_blacklisted === true) {
                $return["errors"][] = l("error_email_blacklisted", "listings_booking");
            } elseif (!empty($is_blacklisted['error'])) {
                $return["errors"][] = l("error_email_incorrect", "users");
            }

            $eparams["where"]["email"] = $return["data"]["email"];
            $this->ci->load->model("users/models/Users_model");
            $count = $this->ci->Users_model->get_users_count($eparams);
            if ($count > 0) {
                $return["errors"][] = l("error_email_already_exists", "users");
            }
        }

        // fname
        if (isset($data['fname'])) {
            $return["data"]["fname"] = trim(strip_tags($data["fname"]));
            if (empty($return["data"]["fname"])) {
                $return["errors"][] = l("error_empty_fname", "users");
            }
        }

        // sname
        if (isset($data['sname'])) {
            $return["data"]["sname"] = trim(strip_tags($data["sname"]));
            if (empty($return["data"]["sname"])) {
                $return["errors"][] = l("error_empty_sname", "users");
            }
        }

        // phone
        if (isset($data['phone'])) {
            $return["data"]["phone"] = trim(strip_tags($data["phone"]));
            if (empty($return["data"]["phone"])) {
                $return["errors"][] = l("error_empty_phone", "users");
            } else {
                $this->ci->load->helper("start");
                $phone_format = get_phone_format('regexp');
                if ($phone_format !== false && !preg_match($phone_format, $return["data"]["phone"])) {
                    $return["errors"][] = l("error_phone_incorrect", "users");
                }
            }
        }

        return $return;
    }

    /**
     * Calculate price of calendar period
     *
     * @param array/integer $listing    listing data/identifier
     * @param string        $date_start date of begin period
     * @param string        $date_end   date of end period
     * @param integer       $guests     number of guests
     *
     * @return mixed
     */
    public function get_period_price($listing, $date_start, $date_end, $guests = 1)
    {
        $value = 0;

        $this->ci->load->model('Listings_model');

        if (!is_array($listing)) {
            $this->ci->Listings_model->set_format_settings('use_format', false);
            $listing = $this->ci->Listings_model->get_listing_by_id($listing);
            $this->ci->Listings_model->set_format_settings('use_format', true);
        }

        if (isset($listing['booking']['periods'])) {
            $periods = $listing['booking']['periods'];
        } else {
            $params = array(
                'listings' => $listing['id'],
                'users' => $listing['id_user'],
                'date_start <=' => $date_end,
                'date_end >' => $date_start,
                'status' => self::STATUS_OPEN,
            );
            $periods = $this->get_periods_list($params);
        }

        $date_start = strtotime($date_start . ' 00:00:00');
        $date_end = strtotime($date_end . ' 00:00:00');

        switch ($listing['price_period']) {
            case self::PRICE_PERIOD_DAY:
                $days = 0;
                foreach ($periods as $period) {
                    if ($period['status'] != self::STATUS_OPEN) {
                        continue;
                    }
                    $period_date_start = strtotime($period['date_start'] . ' 00:00:00');
                    $period_date_end = strtotime($period['date_end'] . ' 00:00:00') + 86400;
                    if ($period_date_start > $date_end || $period_date_end < $date_start) {
                        continue;
                    }
                    $min = max($date_start, $period_date_start);
                    $max = min($date_end, $period_date_end);
                    if ($min != $max) {
                        $days += $periods_days = ($max - $min) / 86400;
                        $value += $period['price'] * $periods_days;
                    }
                }                
                $periods_days = ($date_end - $date_start) / 86400 - $days;

                if ($periods_days > 0) {
                    if ($listing['price_negotiated']) {
                        return 0;
                    }
                    if ($listing['price_reduced'] > 0) {
                        $price_value = $listing['price_reduced'];
                        $value += $periods_days * $price_value;
                    } else {
                        $order_days = ($date_end - $date_start) / 86400;
                        if ($listing['price_month'] > 0 && $order_days >= 28) {
                            $price_value = $listing['price_month'];
                            $value += (integer)($order_days / 28) * $price_value;
                            $order_days = $order_days % 28;
                        } elseif ($listing['price_week'] > 0 && $order_days >= 7) {
                            $price_value = $listing['price_week'];
                            $value += (integer)($order_days / 7) * $price_value;
                            $order_days = $order_days % 7;
                        } else {
                            $price_value = $listing['price'];
                            $value += $order_days * $price_value;
                        }
                    }
                }
                break;
            case self::PRICE_PERIOD_MONTH:
                $month_start = date('m', $date_start);
                $year_start = date('Y', $date_start);

                $month_end = date('m', $date_end);
                $year_end = date('Y', $date_end);

                $months_start = $year_start * 12 + $month_start;
                $months_end = $year_end * 12 + $month_end;

                $months = 0;

                foreach ($periods as $period) {
                    if ($period['status'] != self::STATUS_OPEN) {
                        continue;
                    }
                    $period_date_start = strtotime($period['date_start']);
                    $period_date_end = strtotime($period['date_end']);
                    if ($period_date_start > $date_end || $period_date_end < $date_start) {
                        continue;
                    }

                    $period_month_start = date('m', $period_date_start);
                    $period_year_start = date('Y', $period_date_start);

                    $period_month_end = date('m', $period_date_end);
                    $period_year_end = date('Y', $period_date_end);

                    $period_months_start = $period_year_start * 12 + $period_month_start;
                    $period_months_end = $period_year_end * 12 + $period_month_end;

                    $min = max($months_start, $period_months_start);
                    $max = min($months_end, $period_months_end);
                    if ($min != $max) {
                        $months += $periods_months = ($max - $min) / 86400;
                        $value += $period['price'] * $periods_months;
                    }
                }

                $periods_months = ($months_end - $months_start) - $months;
                if ($periods_months > 0) {
                    if ($listing['price_negotiated']) {
                        return 0;
                    }
                    if ($listing['price_reduced'] > 0) {
                        $price_value = $listing['price_reduced'];
                    } else {
                        $price_value = $listing['price'];
                    }
                    $value += $periods_months * $price_value;
                }
                break;
        }

        switch ($listing['price_type']) {
            case 1:
                break;
            case 2:
                $value *= $guests;
                break;
        }

        return $value;
    }

    /**
     * Clean up booking listings cache by cron
     *
     * @return void
     */
    public function search_booking_cron()
    {
        $this->ci->db->where('expire <=', time());
        $this->ci->db->delete(LISTINGS_BOOKING_CACHE_TABLE);
    }

    public function is_data_changed($period_id, $saved_data, $new_data)
    {
        $fields = $this->fields;

        foreach ($fields as $field) {
            if (isset($new_data[$field])) {
                $saved_value = trim(strip_tags($saved_data[$field]));
                $new_value = trim(strip_tags($new_data[$field]));
                if ($saved_value != $new_value && !empty($new_value)) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Check if user had put guest email into blacklist
     *
     * @param type $email
     */
    public function is_email_blacklisted_by_user($email = '', $id_owner = null)
    {
        $email = trim(strip_tags($email));

        if (!empty($email)) {
            $this->ci->db->select();
            $this->ci->db->where(array(
                'guest_email' => $email,
                'id_owner' => $id_owner,
            ));
            $this->ci->db->from(LISTINGS_BOOKING_BLACKLIST_TABLE);
            $result = $this->ci->db->get()->result_array();
            if (count($result) > 0)
                return true;
        }else {
            $result['error'] = 'Undefined error';
        }

        return $result;
    }

    public function toBlacklist($period_id)
    {
        $this->setPeriodStatus($period_id, self::STATUS_BLACKLISTED);
        $period = $this->getPeriodById($period_id);
        if (!empty($period['guest_email'])) {
            $this->addEmailToBlacklist($period['id_owner'], $period['guest_email']);
            $this->sendBlacklistNotification($period);
        }
        return true;
    }

    private function setPeriodStatus($period_id, $status)
    {
        return $this->ci->db->set('status', $status)
                ->where('id', $period_id)
                ->update(LISTINGS_BOOKING_PERIODS_TABLE);
    }

    private function addEmailToBlacklist($owner_id, $email)
    {
        return $this->ci->db->insert(LISTINGS_BOOKING_BLACKLIST_TABLE, [
                'id_owner' => $owner_id,
                'guest_email' => $email,
        ]);
    }

    /**
     * Delete all requests by guest email
     * @param int $owner_id
     * @param string $email
     */
    public function deletePeriodsByEmail($owner_id, $email)
    {
        $guest_periods = $this->get_periods_list([
            'where' => [
                'guest_email' => $email,
                'id_owner' => $owner_id,
            ],
        ]);
        $guest_periods_ids = [];
        foreach ($guest_periods as $item) {
            $guest_periods_ids[] = $item['id'];
        }
        $this->delete_period($guest_periods_ids);
    }

    /**
     * Send blacklist notification
     *
     * @param array $period
     * @return boolean
     */
    private function sendBlacklistNotification(array $period)
    {
        if (empty($period['guest_email'])) {
            return false;
        }

        if (!empty($period['booking_data']['id_lang'])) {
            $lang_id = $period['booking_data']['id_lang'];
        } else {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        $mail_data = [
            'listing_owner' => $period['user']['output_name'],
            'listing_name' => $period['user']['output_name'],
        ];
        $this->ci->load->model('Notifications_model');
        return $this->ci->Notifications_model->send_notification(
                $period['guest_email'], 'guest_booking_blacklisted', $mail_data, '', $lang_id
        );
    }

}
