<?php

namespace Pg\Modules\Memberships\Models;

/**
 * Memberships module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

define('MEMBERSHIPS_USERS_TABLE', DB_PREFIX . 'memberships_users');

/**
 * User membership model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Memberships
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Memberships_users_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * User membership object properties
     *
     * @var array
     */
    private $fields = array(
        'id',
        'id_user',
        'id_membership',
        'membership_data',
        'services',
        'services_count',
        'is_active',
        'date_activated',
        'date_expired',
    );

    /**
     * Settings for formatting membership object
     *
     * @var array
     */
    protected $format_settings = array(
        'get_user' => false,
    );

    /**
     * Class constructor
     *
     * @return Memberships_users_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Save user membership
     *
     * @param integer $membership_id   membership identifier
     * @param array   $membership_data membership data
     *
     * @return integer
     */
    public function save($membership_id = null, $membership_data = array())
    {
        if (is_null($membership_id)) {
            $this->ci->db->insert(MEMBERSHIPS_USERS_TABLE, $membership_data);
            $membership_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $membership_id);
            $this->ci->db->update(MEMBERSHIPS_USERS_TABLE, $membership_data);
        }

        return $membership_id;
    }

    /**
     * Return user membership by id
     *
     * @param array $user_membership_id
     *
     * @return array|false
     */
    public function getUserMembershipById($user_membership_id)
    {
        if (empty($user_membership_id)) {
            throw new Exception('Empty $user_membership_id');
        }
        $results = $this->ci->db->select(implode(', ', $this->fields))
                ->from(MEMBERSHIPS_USERS_TABLE)
                ->where('id', $user_membership_id)
                ->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $results[0];
        }

        return false;
    }

    /**
     * Return user's memberships as array
     *
     * @param array $order_by          sorting data
     * @param array $params            filter parameters
     * @param array $filter_object_ids membership identifiers for filtering
     *
     * @return array
     */
    public function getUserMembershipsList($order_by = null, $params = array(), $filter_object_ids = null)
    {
        $data = array();

        $this->ci->db->select(implode(", ", $this->fields))->from(MEMBERSHIPS_USERS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            $this->ci->db->where($params["where"]);
        }
        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }
        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }
        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }
        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        $results = $this->ci->db->get()->result_array();

        return $this->formatMemberships($results);
    }

    /**
     * Change settings for listings formatting
     *
     * @param string $name  parameter name
     * @param mixed  $value parameter value
     *
     * @return void
     */
    public function setFormatSettings($name, $value = false)
    {
        if (!is_array($name)) {
            $name = array($name => $value);
        }
        foreach ($name as $key => $item) {
            $this->format_settings[$key] = $item;
        }
    }

    /**
     * Format membership data
     *
     * @param array $membership_data membership data
     *
     * @return array
     */
    public function formatMembership($membership_data)
    {
        return array_shift($this->formatMemberships(array($membership_data)));
    }

    /**
     * Format memberships' data
     *
     * @param array $memberships_data memberships' data
     *
     * @return array
     */
    public function formatMemberships($memberships_data)
    {
        if (!$memberships_data) {
            return array();
        }

        $user_memberships_ids = array();
        $user_memberships_services = array();

        foreach ($memberships_data as $membership_data) {
            $user_memberships_ids[$membership_data['id']] = $membership_data['id_membership'];
        }

        if (!empty($user_memberships_ids)) {
            $this->ci->load->model('Memberships_model');
            $memberships = $this->ci->Memberships_model->getMembershipsList(array('ids' => $user_memberships_ids));
            $memberships = $this->ci->Memberships_model->formatMemberships($memberships);

            $temp = array();
            foreach ($memberships as $membership) {
                $temp[$membership['id']] = $membership;
            }
            foreach ($memberships_data as $key => $membership_data) {
                $memberships_data[$key]['membership_info'] = !empty($temp[$membership_data['id_membership']]) ?
                    $temp[$membership_data['id_membership']] : (array) unserialize($membership_data['membership_data']);
            }
        }

        if (!empty($user_memberships_services)) {
            $param['where_in']['id_users_package'] = $user_memberships_ids;
            $this->ci->load->model('services/models/Services_users_model');
            $services = $this->ci->Services_users_model->get_services_list($param);
            foreach ($user_package_services as $ups) {
                $user_memberships_services[$ups['id_users_membership']][$ups['id']] = $ups;
            }
        }

        return $memberships_data;
    }

    /**
     * Return number of users' memberships
     *
     * @param array $params            filter parameters
     * @param array $filter_object_ids identifiers for filtering
     *
     * @return integer
     */
    public function getUserMembershipsCount($params = array(), $filter_object_ids = null)
    {
        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            $this->ci->db->where($params["where"]);
        }
        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }
        if (isset($params["where_not_in"]) && is_array($params["where_not_in"]) && count($params["where_not_in"])) {
            foreach ($params["where_not_in"] as $field => $value) {
                $this->ci->db->where_not_in($field, $value);
            }
        }
        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }
        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]['cnt']);
        }

        return 0;
    }

    /**
     * Update memberships by cron
     *
     * @return void
     */
    public function cronUpdateMemberships()
    {
        $this->ci->db->select('id')->from(MEMBERSHIPS_USERS_TABLE)->where('date_expired <', time());
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $ids = array();
            foreach ($results as $r) {
                $ids[] = $r['id'];
            }
            if ($ids) {
                $params['where_in']['id_users_membership'] = $ids;
                $this->ci->load->model('services/models/Services_users_model');
                $this->ci->Services_users_model->update_service($params, array('status' => 0));
            }
        }
    }

    /**
     * Remove user memberships
     *
     * @param integer $user_id            user identifier
     * @param integer $user_membership_id user membership identifier
     *
     * @return void
     */
    public function removeUserMemberships($user_id, $user_membership_id)
    {
        $params = array(
            'where' => array(
                'id_user' => $user_id,
                'id !=' => $user_membership_id,
            ),
        );
        $user_memberships = $this->getUserMembershipsList(null, $params);

        if (empty($user_memberships)) {
            return;
        }

        $this->ci->db->where('id_user', $user_id);
        $this->ci->db->where('id !=', $user_membership_id);
        $this->ci->db->delete(MEMBERSHIPS_USERS_TABLE);
    }

}
