<?php

namespace Pg\Modules\Menu\Models;

/**
 * Menu module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('INDICATORS_TABLE')) {
    define('INDICATORS_TABLE', DB_PREFIX . 'menu_indicators');
}
if (!defined('INDICATORS_TYPES_TABLE')) {
    define('INDICATORS_TYPES_TABLE', DB_PREFIX . 'menu_indicators_types');
}

/**
 * Indicators model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Menu
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Indicators_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Indicator properties in data source
     *
     * @var array
     */
    public $fields = array(
        'id',
        'gid',
        'user_id',
        'value',
        'created',
    );

    /**
     * Indicator type properties in data source
     *
     * @var array
     */
    public $types_fields = array(
        'id',
        'gid',
        'auth_type',
        // name_1
        // name_2
        // ...
    );

    /**
     * Types of authorization
     *
     * @var array
     */
    protected $_auth_types = array('user', 'admin', 'mudule');

    /**
     * Constructor
     *
     * @return Indicators_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();

        $langs = (array) $this->pg_language->languages;
        foreach ($langs as $lang) {
            $this->types_fields[] = 'name_' . $lang['id'];
        };
    }

    /**
     * Save indicator objects to data source.
     *
     * Depends on "use_indicators" menu setting.
     *
     * @param string  $gid     type GUID
     * @param integer $user_id user identifier
     * @param string  $uid     unique object identifier
     * @param string  $value   indicator valie
     *
     * @return object/boolean
     */
    public function add($gid, $uid = '', $user_id = 0, $value = '')
    {
        if (!$this->ci->pg_module->get_module_config('menu', 'use_indicators')) {
            return false;
        }

        // Delete old indicator
        if (!empty($uid)) {
            $this->delete($gid, $uid, true);
        }

        $data = array(
            'gid' => $gid,
            'value' => $value,
            'uid' => $uid,
            'user_id' => $user_id,
        );

        $this->ci->db->insert(INDICATORS_TABLE, $data);

        return $this;
    }

    /**
     * Return menu indicators as array
     *
     * @param string  $auth_type authorization type (admin | module | user)
     * @param integer $user_id   user identifier
     *
     * @return array
     */
    public function get($auth_type = null, $user_id = null)
    {
        if ('user' === $auth_type && empty($user_id)) {
            log_message('ERROR', '(indicators) Empty user id');

            return false;
        }
        $this->ci->db->select('i.gid')->from(INDICATORS_TABLE . ' AS i');
        if (!empty($auth_type)) {
            $this->ci->db->join(INDICATORS_TYPES_TABLE . ' AS t', 't.gid = i.gid')
                ->where('t.auth_type', $auth_type);
            if ('user' === $auth_type) {
                $this->ci->db->where('i.user_id', $user_id);
            };
        }; // else get all indicators
        $result = $this->ci->db->get()->result_array();
        if (empty($result)) {
            return array();
        };
        $indicators = array();
        foreach ($result as $indicator) {
            if (!empty($indicators[$indicator['gid']])) {
                ++$indicators[$indicator['gid']];
            } else {
                $indicators[$indicator['gid']] = 1;
            }
        };

        return $indicators;
    }

    /**
     * Removes indicators for current user
     *
     * @param string  $gid type GUID
     * @param string  $uid unique object identifier
     * @param boolean $all if true, session data will be ignored
     *
     * @return object
     */
    public function delete($gid, $uid = null, $all = false)
    {
        if (!$this->ci->pg_module->get_module_config('menu', 'use_indicators')) {
            return $this;
        } elseif (empty($gid)) {
            log_message('ERROR', '(indicators) Empty indicator gid');

            return $this;
        };

        if (!$all) {
            $auth_type = $this->ci->session->userdata('auth_type');
            if ('user' === $auth_type) {
                $this->ci->db->where('user_id', $this->ci->session->userdata('user_id'))
                    ->where('gid', $gid);
                if (!empty($uid)) {
                    $this->ci->db->where_in('uid', $uid);
                }
                $this->ci->db->delete(INDICATORS_TABLE);
            } else {
                // CI's active record can't handle such queries
                $query = 'DELETE i FROM ' . INDICATORS_TABLE . ' i
					JOIN ' . INDICATORS_TYPES_TABLE . " t ON t.gid = i.gid
					WHERE t.auth_type = '$auth_type' AND i.gid = '$gid'";
                if (!empty($uid)) {
                    $uid = (array) $uid;
                    foreach ($uid as $key => $value) {
                        $uid[$key] = $this->ci->db->escape($value);
                    }
                    $query .= ' AND uid IN (' . implode(',', $uid) . ')';
                }
                $this->db->query($query);
            }
        } else {
            if (!empty($uid)) {
                $this->ci->db->where_in('uid', $uid);
            }
            $this->ci->db->where('gid', $gid)
                ->delete(INDICATORS_TABLE);
        }

        return $this;
    }

    /**
     * Remove indicators by cron
     *
     * @return void
     */
    public function delete_old()
    {
        $lifetime = (int) $this->ci->pg_module->get_module_config('menu', 'indicator_lifetime');
        if ($lifetime < 1) {
            $lifetime = 24;
        };
        $query = 'DELETE i FROM ' . INDICATORS_TABLE . ' i
					JOIN ' . INDICATORS_TYPES_TABLE . " t ON t.gid = i.gid
					WHERE t.delete_by_cron = '1'
					AND i.user_id = '0'
					AND i.created < DATE_SUB(NOW(), INTERVAL $lifetime hour)";
        $this->db->query($query);
        echo 'Old indicators clean : Old indicators was remove successfully';
    }

    /**
     * Saves indicator type object to data source
     *
     * @param string $gid  type GUID
     * @param array  $data type data
     *
     * @return boolean
     */
    public function save_type($gid = null, $data = array())
    {
        if (empty($data)) {
            log_message('ERROR', '(indicators) Empty indicator data');
            return $this;
        } elseif (!empty($data['auth_type']) && !in_array($data['auth_type'], $this->_auth_types)) {
            log_message('ERROR', '(indicators) Wrong indicator auth type');

            return $this;
        }
        if (isset($data['delete_by_cron'])) {
            $data['delete_by_cron'] = (int) $data['delete_by_cron'];
        } else {
            $data['delete_by_cron'] = 0;
        }
        if (empty($gid)) {
            $this->ci->db->ignore()->insert(INDICATORS_TYPES_TABLE, $data);
        } else {
            $this->ci->db->where('gid', $gid);
            $this->ci->db->update(INDICATORS_TYPES_TABLE, $data);
        }

        return $this;
    }

    /**
     * Returns types of indicators objects as array
     *
     * Depends on "use_indicators" menu setting.
     *
     * @param $params sql criteria for query to data source
     *
     * @return array
     */
    public function get_types($params = null)
    {
        if (!$this->ci->pg_module->get_module_config('menu', 'use_indicators')) {
            return false;
        }
        $this->ci->db->select(implode(', ', $this->types_fields))
            ->from(INDICATORS_TYPES_TABLE);

        if (!empty($params)) {
            if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
                foreach ($params['where'] as $field => $value) {
                    $this->ci->db->where($field, $value);
                }
            };

            if (isset($params['where_in']) && is_array($params['where_in']) && count($params['where_in'])) {
                foreach ($params['where_in'] as $field => $value) {
                    $this->ci->db->where_in($field, $value);
                }
            };

            if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
                foreach ($params['where_sql'] as $value) {
                    $this->ci->db->where($value, null, false);
                }
            };
        };

        $result = $this->ci->db->get()->result_array();
        $types = array();
        $cur_lang = $this->ci->pg_language->current_lang_id;
        foreach ($result as $set) {
            $types[$set['gid']] = $set;
            $types[$set['gid']]['name'] = $types[$set['gid']]['name_' . $cur_lang];
        };

        return $types;
    }

    /**
     * Remove indicator type object by GUID
     *
     * @param string $gid type GUID
     *
     * @return boolean
     */
    public function delete_type($gid)
    {
        if (empty($gid)) {
            log_message('ERROR', '(indicators) Empty type gid');

            return false;
        };
        $this->ci->db->where('gid', $gid)
            ->delete(INDICATORS_TYPES_TABLE);
        $this->delete($gid, null, true);

        return true;
    }

    /**
     * Adds indicator's properties depended on language
     *
     * @param integer $lang_id language identifier
     *
     * @return boolean
     */
    public function lang_dedicate_module_callback_add($lang_id = false)
    {
        if (!$lang_id) {
            return false;
        };
        $fields['name_' . $lang_id] = array(
            'type' => 'VARCHAR',
            'constraint' => '255',
            'null' => false,
        );
        // Add column
        $this->ci->load->dbforge();
        $this->ci->dbforge->add_column(INDICATORS_TYPES_TABLE, $fields);

        return true;
    }

    /**
     * Removes indicator's properties depended on language
     *
     * @param integer $lang_id language identifier
     *
     * @return boolean
     */
    public function lang_dedicate_module_callback_delete($lang_id = false)
    {
        if (!$lang_id) {
            return false;
        };
        $field_name = 'name_' . $lang_id;

        // Delete column
        $fields_exists = $this->ci->db->list_fields(INDICATORS_TYPES_TABLE);
        if (in_array($field_name, $fields_exists)) {
            $this->ci->load->dbforge();
            $this->ci->dbforge->drop_column(INDICATORS_TYPES_TABLE, $field_name);
        };
    }

    /**
     * Import languages to indicators
     *
     * @param array $data       indicator's data
     * @param array $langs_file data by languages
     * @param array $langs_ids  languages identifiers
     *
     * @return boolean
     */
    public function update_langs($data, $langs_file, $langs_ids = null)
    {
        if (empty($data)) {
            log_message('ERROR', '(indicators) Empty data for update');

            return false;
        } elseif (empty($langs_file)) {
            log_message('ERROR', '(indicators) Empty langs for update');

            return false;
        };

        // Get all langs if $langs_ids is empty
        if (empty($langs_ids)) {
            $langs = (array) $this->pg_language->languages;
            foreach ($langs as $lang) {
                $langs_ids[] = $lang['id'];
            };
        };

        $default_lang = $this->ci->pg_language->get_default_lang_id();
        foreach ($data as $indicator) {
            $langs_data = array();
            foreach ($langs_ids as $lang_id) {
                if (!empty($langs_file[$indicator['gid']][$lang_id])) {
                    $name = $langs_file[$indicator['gid']][$lang_id];
                } elseif (!empty($langs_file[$indicator['gid']][$default_lang])) {
                    // Use default lang
                    $name = $langs_file[$indicator['gid']][$default_lang];
                } else {
                    continue;
                };
                $langs_data['name_' . $lang_id] = $name;
            };
            if (!empty($langs_data)) {
                $this->save_type($indicator['gid'], $langs_data);
            };
        };

        return true;
    }

    /**
     * Exports languages of indicators
     *
     * @param array $data      indicator's data
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function export_langs($data, $langs_ids = null)
    {
        if (empty($data)) {
            log_message('ERROR', '(indicators) Empty data for export');
        };
        $gids = array();
        foreach ($data as $indicator) {
            $gids[] = $indicator['gid'];
        };

        $indicators = $this->get_types(array('where_in' => array('gid' => $gids)));

        // Get all langs if $langs_ids is empty
        if (empty($langs_ids)) {
            $langs = (array) $this->pg_language->languages;
            foreach ($langs as $lang) {
                $langs_ids[] = $lang['id'];
            };
        };

        $langs_data = array();
        foreach ($indicators as $indicator) {
            foreach ($langs_ids as $lang_id) {
                if (!empty($indicator['name_' . $lang_id])) {
                    $langs_data[$indicator['gid']][$lang_id] = $indicator['name_' . $lang_id];
                };
            };
        };

        return $langs_data;
    }

}
